package com.hisschemoller.controller 
{
	import com.hisschemoller.utils.Conversions;
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.AudioEngineProxy;
	import com.hisschemoller.model.enum.FileDataType;
	import com.hisschemoller.model.vo.AlertVO;
	import com.hisschemoller.utils.audio.processors.IAudioInstrument;
	import com.hisschemoller.utils.audio.vo.SoundVO;
	import com.hisschemoller.utils.delegates.AudioDecoderDelegate;
	import com.hisschemoller.utils.delegates.AudioLoaderDelegate;
	import com.hisschemoller.utils.delegates.AudioOpenDelegate;
	import com.hisschemoller.utils.delegates.XMLLoaderDelegate;
	import com.hisschemoller.utils.delegates.vo.AudioLoaderVO;
	import com.hisschemoller.utils.midi.MidiProgramChangeVO;
	
	import org.puremvc.as3.interfaces.ICommand;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.command.SimpleCommand;
	
	import mx.rpc.Responder;
	
	import flash.net.FileReference;	
	/**
	 * @author wouter.hisschemoller
	 * (c) Lost Boys Feb 2, 2010
	 */
	public class FileDataCommand extends SimpleCommand implements ICommand 
	{
		private var _fileDataType : String;
		private var _xml : XML;
		private var _decodedStack : Vector.<AudioLoaderVO>;

		override public function execute(notification : INotification) : void
		{
			_fileDataType = notification.getType();
			
			switch(_fileDataType)
			{
				case FileDataType.STARTUP_XML_FILE:
					new XMLLoaderDelegate(new Responder(xmlResult, xmlFault)).loadXML(String(notification.getBody()));
					break;
					
				case FileDataType.OPEN_LOCAL_AUDIO_FILE:
					_xml = XML(notification.getBody());
					new AudioOpenDelegate(new Responder(openResult, openFault)).openAudioFile();
					break;
			}
		}
		
		/**
		 * XML succesfully loaded.
		 */
		public function xmlResult(xml : XML) : void
		{
			parseFileURLsFromXML(xml);
		}
		
		/**
		 * XML failed to load.
		 */
		public function xmlFault(alertVO : AlertVO) : void
		{
			facade.sendNotification(AppFacade.ALERT, alertVO);
		}
		
		/**
		 * Audio file succesfully opened.
		 */
		public function openResult(fileReference : FileReference) : void
		{
			/** Create an AudioLoadData object for the injected file. */
			var audioLoaderVO : AudioLoaderVO = new AudioLoaderVO(fileReference.name);
			audioLoaderVO.byteArray = fileReference.data;
			audioLoaderVO.fileExtention = fileReference.name.substr(fileReference.name.length - 3).toLowerCase();
			
			/** Send to loadResult where it will be sent to the decoder. */
			loadResult(Vector.<AudioLoaderVO>([audioLoaderVO]));
		}
		
		/**
		 * Audio file failed to open.
		 */
		public function openFault(alertVO : AlertVO) : void
		{
			facade.sendNotification(AppFacade.ALERT, alertVO);
		}

		/**
		 * Get all the audio file urls from the XML.
		 */
		private function parseFileURLsFromXML(xml : XML) : void
		{
			XML.ignoreWhitespace = true;
			_xml = xml;
			
			var xmlList : XMLList = xml..sound;
			var fileUrls : Vector.<String> = new Vector.<String>();
			
			for(var i : uint = 0;i < xmlList.length(); i++)
			{
				fileUrls.push(xmlList[i].@url);
			}
			
			new AudioLoaderDelegate(new Responder(loadResult, null), facade).loadAudioFiles(fileUrls);
		}
		
		/**
		 * Audio files succesfully loaded.
		 */
		public function loadResult(loadedStack : Vector.<AudioLoaderVO>) : void
		{
			new AudioDecoderDelegate(new Responder(decodeResult, null), facade).decodeAudio(loadedStack);
		}
		
		/**
		 * Audio files succesfully decoded.
		 */
		public function decodeResult(decodedStack : Vector.<AudioLoaderVO>) : void
		{
			_decodedStack = decodedStack;
			
			switch(_fileDataType)
			{
				case FileDataType.STARTUP_XML_FILE:
					parseStartupXML(_xml);
					break;
					
				case FileDataType.OPEN_LOCAL_AUDIO_FILE:
					addNewAudioFile();
					break;
			}
		}

		/**
		 * Parse the XML, create objects and store them in the proxy.
		 */
		private function parseStartupXML(xml : XML) : void
		{
			/** Parse the sounds. */
			var xmlList : XMLList = xml..sound;
			var sounds : Vector.<SoundVO> = new Vector.<SoundVO>();
			
			for(var i : uint = 0;i < xmlList.length(); i++)
			{
				var audioLoaderVO : AudioLoaderVO = getAudioDataByURL(xmlList[i].@url, _decodedStack);
				_decodedStack.splice(_decodedStack.indexOf(audioLoaderVO), 1);
				
				var soundVO : SoundVO = new SoundVO();
				soundVO.parseXML(xmlList[i]);
				soundVO.byteArray = audioLoaderVO.byteArray;
				soundVO.channels = audioLoaderVO.channels;
				soundVO.numSamplesTotal = audioLoaderVO.numSamplesTotal;
				sounds.push(soundVO);
				
				createSound(soundVO);
			}
			
			/** Setup the instruments for this particular application. */
			facade.sendNotification(AppFacade.STARTUP_INSTRUMENTS, xml..instrument);
			
			/** Tell the system all startup work is done. */
			facade.sendNotification(AppFacade.STARTUP_DONE);
		}

		/**
		 * Put the audio ByteArray in a SoundVO object and send away to store in sound bank.
		 */
		public function addNewAudioFile() : void
		{
			if(_decodedStack.length == 0)
			{
				facade.sendNotification(AppFacade.ALERT, new AlertVO("Error", "No audio was decoded."));
				return;
			}
			else if(_decodedStack.length > 1)
			{
				facade.sendNotification(AppFacade.ALERT, new AlertVO("Warning", "Unused Audio remains in the decoded stack."));
			}
			
			var audioLoaderVO : AudioLoaderVO = _decodedStack.splice(0, 1)[0];
			
			var soundVO : SoundVO = new SoundVO();
			soundVO.id = audioLoaderVO.url;
			soundVO.url = audioLoaderVO.url;
			soundVO.byteArray = audioLoaderVO.byteArray;
			soundVO.channels = audioLoaderVO.channels;
			soundVO.numSamplesTotal = audioLoaderVO.numSamplesTotal;
			
			createSound(soundVO);
			
			var programXML : XML = _xml..program[0];
			programXML.@id = "program" + new Date().getTime();
			programXML.@soundid = soundVO.id;
			programXML.@start = 0;
			programXML.@end = soundVO.numSamplesTotal;
			
			var instrument : IAudioInstrument = AudioEngineProxy(facade.retrieveProxy(AudioEngineProxy.NAME)).getInstrumentByID(_xml.@id);
			
			if(instrument)
			{
				var programIndex : int = instrument.addProgram(programXML);
				
				/** If the program was successfully added select the program. */
				if(programIndex > -1)
				{
					var midiProgramChangeVO : MidiProgramChangeVO = new MidiProgramChangeVO();
					midiProgramChangeVO.number = programIndex;
					
					instrument.programChange(midiProgramChangeVO);
				}
			}
		}

		/**
		 * Get sound data for specified sound
		 */
		private function getAudioDataByURL(url : String, stack : Vector.<AudioLoaderVO>) : AudioLoaderVO 
		{
			for each (var audioLoadData : AudioLoaderVO in stack)
			{
				if(audioLoadData.url == url) return audioLoadData;
			}

			return null;
		}
		
		/**
		 * 
		 */
		private function createSound(soundVO : SoundVO) : void
		{
			soundVO.samples = Conversions.byteArrayToVector(soundVO.byteArray, soundVO.channels);
			
			facade.sendNotification(AppFacade.ADD_SOUND, soundVO);
		}
	}
}
