package com.hisschemoller.controller 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.AudioEngineProxy;
	import com.hisschemoller.model.WaveDisplayProxy;
	import com.hisschemoller.model.WaveEditorProxy;
	import com.hisschemoller.model.vo.AlertVO;
	import com.hisschemoller.utils.audio.enum.AudioProcessorType;
	import com.hisschemoller.view.WaveDisplayMediator;

	import org.puremvc.as3.interfaces.ICommand;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.command.SimpleCommand;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Feb 23, 2010
	 */
	public class StartupInstrumentsCommand extends SimpleCommand implements ICommand 
	{
		override public function execute(notification : INotification) : void
		{
			var instrumentsXmlList : XMLList = XMLList(notification.getBody());
			
			/** In the case of this Audio Editor there should only be one instrument. */
			if(instrumentsXmlList.length() != 1)
			{
				facade.sendNotification(AppFacade.ALERT, new AlertVO("Config XML Error", "There are " + instrumentsXmlList.length() + " instruments defined in the config file. There should be only one instrument. Please correct the settings file."));
				return;
			}
			
			var audioEngine : AudioEngineProxy = AudioEngineProxy(facade.retrieveProxy(AudioEngineProxy.NAME));
			
			/** Loop through all instruments */
			var n : uint = instrumentsXmlList.length();
			for(var i : uint = 0;i < n; i++)
			{
				var instrumentXML : XML = instrumentsXmlList[i];
				
				/** Only PITCHED_SAMPLE type will work in this app. */
				if(instrumentXML.@type != AudioProcessorType.LOOPED_SAMPLE_PROCESSOR)
				{
					facade.sendNotification(AppFacade.ALERT, new AlertVO("Wrong type", "An instrument defined in the settings file is of an unsupported type: " + instrumentXML.@type + ". Please correct the settings file."));
					return;
				}
				
				/** The instrument proxy. */
				facade.registerProxy(new WaveDisplayProxy(instrumentXML.@id + WaveDisplayMediator.NAME_SUFFIX));
				facade.registerProxy(new WaveEditorProxy(instrumentXML.@id, instrumentXML));
				
				/** Add the instrument to the audio engine. */
				audioEngine.addInstrument(WaveEditorProxy(facade.retrieveProxy(instrumentXML.@id)));
				
				/** Send note to add the instrument (wave editor) to the app stage and create the mediators. */
				facade.sendNotification(AppFacade.ADD_INSTRUMENT, instrumentXML.@id);
			}
		}
	}
}
