package com.hisschemoller.model 
{
	import com.hisschemoller.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.utils.audio.vo.SampleVO;

	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;

	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.SampleDataEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.utils.getTimer;
	import flash.utils.setTimeout;
	/**
	 * @author Wouter Hisschemöller
	 * (c) Jul 12, 2009
	 */
	public class AudioDriverProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = "AudioDriverProxy";
		public static const ENTER_BUFFER : String = "enterBuffer";
		public static const ENTER_FRAME : String = "enterFrame";
		public static const BUFFER_BLOCK_SIZE : int = 3072;
		public static const BUFFER_MULTIPLIER : int = 20;
		public static const BUFFER_TOTAL_SIZE : int = BUFFER_BLOCK_SIZE * BUFFER_MULTIPLIER;
		public static const SAMPLE_RATE : int = 44100;
		public static const BYTES_PER_MONO_SAMPLE : int = 4;
		public static const BUFFER_DURATION_MS : Number = (BUFFER_BLOCK_SIZE / SAMPLE_RATE) * 1000;
		private const SOUND : Sound = new Sound();
		private var _buffer : Vector.<SampleVO>;
		private var _soundChannel : SoundChannel;
		private var _latencyInMS : Number = 0;
		private var _latencyInSamples : int = 0;
		private var _engine : AudioEngineProxy;
		private var _output : AudioOutputProxy;
		private var _stage : Stage;
		private var _streamInfo : AudioStreamInfoVO = new AudioStreamInfoVO();
		private var _lastBuffer : int;
		private var _bufferStartIndex : int;
		private var _bufferProcessTime : Number;
		private var _frameTime : Number;
		private var _isPlaying : Boolean;

		public function AudioDriverProxy(stage : Stage)
		{
			_stage = stage;
			
			super(NAME, data);
			
			_buffer = new Vector.<SampleVO>(BUFFER_TOTAL_SIZE, true);
			
			var i : int = -1;
			var n : uint = BUFFER_TOTAL_SIZE;
			while(++i < n)
			{
				_buffer[i] = new SampleVO();
			}
			
			SOUND.addEventListener(SampleDataEvent.SAMPLE_DATA, sampleDataHandler);
		}
		
		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_engine = facade.retrieveProxy(AudioEngineProxy.NAME) as AudioEngineProxy;
			_output = facade.retrieveProxy(AudioOutputProxy.NAME) as AudioOutputProxy;
			
			/** Delay 500ms to avoid click noise at startup. */
			setTimeout(start, 500);
		}

		/**
		 * 
		 */
		public function play() : void
		{
			_isPlaying = true;
		}

		/**
		 * 
		 */
		public function stop() : void
		{
			_isPlaying = false;
		}

		/**
		 * 
		 */
		private function start() : void
		{
			_lastBuffer = getTimer();
			_frameTime = new Date().getTime();
			_soundChannel = SOUND.play();
			
			_stage.addEventListener(Event.ENTER_FRAME, enterFrameHandler);
		}

		/**
		 * 
		 */
		private function sampleDataHandler(event : SampleDataEvent) : void
		{
			if(_soundChannel)
			{
				/** Latency in milliSeconds. This can vary and needs to be calculated each SampleDataEvent. */
				/** The formula is ((SampleDataEvent.position / 44.1) - SoundChannelObject.position) */
				_latencyInMS = (event.position * 2.267573696145e-02) - _soundChannel.position;
				_latencyInSamples = int(_latencyInMS * SAMPLE_RATE * 0.001);
			}
			
			if(_isPlaying) facade.sendNotification(ENTER_BUFFER);
			
			/** Set the write position to the next block in the buffer. */
			_bufferStartIndex += BUFFER_BLOCK_SIZE;
			if(_bufferStartIndex >= _buffer.length) _bufferStartIndex = 0;
			
			clearBuffer();
			
			_engine.process(_buffer, _bufferStartIndex);
			_output.process(_buffer, _bufferStartIndex);

			//var l : Vector.<Number> = BUFFER[0]; /** left channel */
			//var r : Vector.<Number> = BUFFER[1]; /** right channel */
			var i : int = _bufferStartIndex;
			var n : int = _bufferStartIndex + BUFFER_BLOCK_SIZE;
			
			for(;i < n; ++i)
			{
				event.data.writeFloat(_buffer[i].left);
				event.data.writeFloat(_buffer[i].right);
			}
			
			/** Store the time at which the latest buffer was processed. */
			_bufferProcessTime = new Date().getTime();
		}

		/**
		 * Set all samples in the buffer to zero (= silence).
		 */
		private function clearBuffer() : void
		{
			var i : int = _bufferStartIndex;
			var n : int = _bufferStartIndex + BUFFER_BLOCK_SIZE;
			
			for(;i < n; ++i)
			{
				_buffer[i].left = 0.0;
				_buffer[i].right = 0.0;
			}
		}
		
		/**
		 * 
		 */
		private function enterFrameHandler(event : Event) : void
		{
			if(!_isPlaying) return;
			
			/** Get time elapsed and number of samples played since the last frame. */
			var now : Number = new Date().getTime();
			var frameDurationInMS : Number = now - _frameTime;
			var frameDurationInSamples : int = frameDurationInMS * SAMPLE_RATE * 0.001;
			_frameTime = now;
			
			/** bufferPlayedMS : How much of the current buffer has played. In milliSeconds. */
			var bufferPlayedMS : Number = now - _bufferProcessTime;
			
			/** bufferPlayedSamples : How much of the current buffer has played. In samples.  */
			var bufferPlayedSamples : int = (bufferPlayedMS / BUFFER_DURATION_MS) * BUFFER_BLOCK_SIZE;
			
			/** Current write index in the buffer. */
			var positionInBuffer : int = _bufferStartIndex + bufferPlayedSamples;
			
			/** Index in the buffer that is now played by the soundcard. */
			var positionWithLatency : int = positionInBuffer - _latencyInSamples;
			if(positionWithLatency < 0) positionWithLatency += BUFFER_TOTAL_SIZE;
			
			_streamInfo.latencyInMS = _latencyInMS;
			_streamInfo.peakLevelLeft = 0;
			_streamInfo.peakLevelRight = 0;
			
			/** Get the audio peak levels. (Polled, so not exact.) */
			var pollInterval : int = frameDurationInSamples / 100; /** Poll 100 times. */
			var pollPosition : int = positionWithLatency; /** Start at the actual sounding position. */
			
			for(var i : int = 0; i < 100; ++i)
			{
				_streamInfo.peakLevelLeft = Math.max(_streamInfo.peakLevelLeft, _buffer[pollPosition].left);
				_streamInfo.peakLevelRight = Math.max(_streamInfo.peakLevelRight, _buffer[pollPosition].right);
				
				pollPosition = (pollPosition + pollInterval) % BUFFER_TOTAL_SIZE;
			}
			
			facade.sendNotification(ENTER_FRAME, _streamInfo);
		}
	}
}
