package com.hisschemoller.model 
{
	import de.popforge.parameter.MappingBoolean;
	import de.popforge.parameter.MappingIntLinear;
	import de.popforge.parameter.MappingNumberLinear;

	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.vo.AlertVO;
	import com.hisschemoller.utils.Conversions;
	import com.hisschemoller.utils.audio.enum.AudioProcessorType;
	import com.hisschemoller.utils.audio.processors.IAudioExporter;
	import com.hisschemoller.utils.audio.processors.IAudioInstrument;
	import com.hisschemoller.utils.audio.processors.IAudioProcessor;
	import com.hisschemoller.utils.audio.processors.LoopedSampleProcessor;
	import com.hisschemoller.utils.audio.vo.LoopedSampleProgramVO;
	import com.hisschemoller.utils.audio.vo.SampleVO;
	import com.hisschemoller.utils.audio.vo.SoundVO;
	import com.hisschemoller.utils.midi.MidiNoteOffVO;
	import com.hisschemoller.utils.midi.MidiNoteOnVO;
	import com.hisschemoller.utils.midi.MidiProgramChangeVO;
	import com.hisschemoller.utils.midi.MidiSpecification;
	import com.hisschemoller.utils.parameter.Parameter;

	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 26, 2010
	 */
	public class WaveEditorProxy extends Proxy implements IProxy, IAudioInstrument, IAudioExporter
	{
		public static const DISPLAY_HORIZONTAL_ZOOM_CHANGED : String = 'displayHorizontalZoomChanged';
		public static const DISPLAY_VERTICAL_ZOOM_CHANGED : String = 'displayVerticalZoomChanged';
		public static const DISPLAY_POSITION_CHANGED : String = 'displayPositionChanged';
		public static const DISPLAY_UPDATE_HORIZONTAL_ZOOM : String = 'displayUpdateHorizontalZoom';
		public static const DISPLAY_UPDATE_VERTICAL_ZOOM : String = 'displayUpdateVerticalZoom';
		public static const DISPLAY_UPDATE_POSITION : String = 'displayUpdatePosition';
		public static const PROGRAM_START_CHANGED : String = "programStartChanged";
		public static const PROGRAM_END_CHANGED : String = "programEndChanged";
		public static const PROGRAM_TRANSPOSE_CHANGED : String = "programTransposeChanged";
		public static const PROGRAM_TUNE_CHANGED : String = "programTuneChanged";
		public static const PROGRAM_REVERSE_CHANGED : String = "programReverseChanged";
		public static const PROGRAM_LOOP_CHANGED : String = "programLoopChanged";
		public static const PROGRAM_SOUND_CHANGED : String = "programSoundChanged";
		private var _processors : Vector.<LoopedSampleProcessor> = new Vector.<LoopedSampleProcessor>();
		private var _programs : Vector.<LoopedSampleProgramVO> = new Vector.<LoopedSampleProgramVO>(MidiSpecification.MAX_VALUE_7BIT, true);
		private var _selectedProgramIndex : int;
		private var _soundVO : SoundVO;
		private var _transpose : Parameter = new Parameter(new MappingIntLinear(-12, 12), 0);
		private var _tune : Parameter = new Parameter(new MappingNumberLinear(-0.5, 0.5), 0);
		private var _reverse : Parameter = new Parameter(new MappingBoolean(), 0);
		private var _loop : Parameter = new Parameter(new MappingBoolean(), 0);
		private var _start : int;
		private var _end : int;

		public function WaveEditorProxy(name : String, data : Object)
		{
			super(name, data);
		}

		/**
		 * Called by the Model when the Proxy is registered
		 */ 
		override public function onRegister() : void
		{
			var instrumentXml : XML = data as XML;
			
			/** Check for the right type. Although wrong types should already be filtered out. */
			if(instrumentXml.@type != AudioProcessorType.LOOPED_SAMPLE_PROCESSOR) return;
			
			var programXMLList : XMLList = instrumentXml..program;
			var n : uint = programXMLList.length();
			while(--n > -1)
			{
				addProgram(programXMLList[n]);
			}
			
			_selectedProgramIndex = int(instrumentXml.@selectedprogram);
		}

		/**
		 * Called by the mediator when it's registered.
		 */
		public function mediatorRegistered() : void
		{
			var midiProgramChangeVO : MidiProgramChangeVO = new MidiProgramChangeVO();
			midiProgramChangeVO.number = _selectedProgramIndex;
			
			programChange(midiProgramChangeVO);
		}
		
		/**
		 * For the audio engine to identify instruments.
		 */
		public function get id() : String
		{
			return getProxyName();
		}

		/**
		 * Parse program XML, create programVO and add to _programs.
		 * @param programXML: XML program definition.
		 * @return int: Index of program slot.
		 */
		public function addProgram(programXML : XML) : int
		{
			/** Check for the right program type. */
			if(programXML.@type != AudioProcessorType.LOOPED_SAMPLE_PROCESSOR) return -1;
			
			/** Create the program. */
			var programVO : LoopedSampleProgramVO = new LoopedSampleProgramVO();
			programVO.id = programXML.@id;
			programVO.type = programXML.@type;
			programVO.reverse = (programXML.@reverse == "true");
			programVO.loop = (programXML.@loop == "true");
			programVO.transpose = programXML.@transpose;
			programVO.tune = programXML.@tune;
			programVO.soundData = SoundBankProxy(facade.retrieveProxy(SoundBankProxy.NAME)).getSoundByID(programXML.@soundid);
			programVO.start = (String(programXML.@start).length != 0) ? int(programXML.@start) : 0;
			programVO.end = (String(programXML.@end).length != 0) ? int(programXML.@end) : programVO.soundData.numSamplesTotal;
			
			/** Determine location to store the program. */
			var programIndex : int = -1;
			if(programXML.@index && programXML.@index >= 0 && programXML.@index < _programs.length)
			{
				/** A valid index was found. */
				programIndex = programXML.@index;
			}
			else
			{
				/** No valid index found. Search first free slot. */
				var n : uint = _programs.length;
				for(var i : uint = 0;i < n; i++)
				{
					if(_programs[n] == null)
					{
						programIndex = n;
						break;
					}
				}
			}
			
			if(programIndex > -1)
			{
				_programs[programIndex] = programVO;
				
				log("n Program \"" + programVO.id + "\" added to instrument " + getProxyName() + ".");
			}
			else
			{
				facade.sendNotification(AppFacade.ALERT, new AlertVO("Program not loaded", "Program " + programVO.id + " couldn't be loaded in instrument " + getProxyName() + " because all 128 program slots are in use."));
			}			
			
			return programIndex;
		}
		
		/**
		 * Save an audio file using the current settings.
		 */
		public function exportAudioFile() : void
		{
			/** If there's no program there's nothing to save. */
			if(_selectedProgramIndex == -1)
			{
				facade.sendNotification(AppFacade.ALERT, new AlertVO("Unable to save", "There's no program to save as an audio file."));
				return;
			}
			
			/** The number of samples needed with transpose, tune, start and end applied. */
			var rate : Number = Conversions.semitoneToRate(_transpose.value + _tune.value);
			var numSamples : uint = (_end - _start) / rate;
			
			/** Create vector to hold the samples. */
			var soundVO : SoundVO = new SoundVO();
			soundVO.samples = new Vector.<SampleVO>(numSamples, true);
			soundVO.channels = _soundVO.channels;
			
			/** Fill the new vector with SampleVO objects. */
			var i : int = -1;
			while(++i < numSamples)
			{
				soundVO.samples[i] = new SampleVO();
			}
			
			/** Let a processor fill the vector with sample data. */
			var processor : LoopedSampleProcessor = new LoopedSampleProcessor(_soundVO.samples, _start, _end, _loop.value);
			processor.rate = rate;
			processor.process(soundVO.samples, 0, numSamples);
			
			facade.sendNotification(AppFacade.FILE_SAVE, soundVO);
		}

		/**
		 * Let each processor add it's data to the buffer.
		 * @param buffer : stereo buffer of AudioDriver.BUFFER_SIZE number of samples
		 * @param bufferStartIndex: From where in the buffer to start to write.
		 */
		public function process(buffer : Vector.<SampleVO>, bufferStartIndex : uint) : void
		{
			var n : int = _processors.length;
			while(--n > -1)
			{
				/** IAudioProcessor returns true if the sound is finished. */
				if(_processors[n].process(buffer, bufferStartIndex, AudioDriverProxy.BUFFER_BLOCK_SIZE))
				{
					facade.sendNotification(AppFacade.PROCESSOR_REMOVED, _processors[n], getProxyName());
					
					/** Remove the event from the _processors list. */
					_processors.splice(n, 1);
				}
			}
		}
		
		/**
		 * Play a sound.
		 */
		public function noteOn(data : MidiNoteOnVO) : void
		{
			var processor : LoopedSampleProcessor = new LoopedSampleProcessor(_soundVO.samples, _start, _end, _loop.value, data.deltaTime, data.pitch, data.velocity);
			processor.rate = Conversions.semitoneToRate(_transpose.value + _tune.value);
			
			addProcessor(processor);
		}
		
		/**
		 * Stop a sound.
		 */
		public function noteOff(data : MidiNoteOffVO) : void
		{
			var n : int = _processors.length;
			while(--n > -1)
			{
				if(_processors[n].pitch == data.pitch) _processors[n].stop();
			}
		}

		/**
		 * Stop all sounds.
		 */
		public function allNotesOff() : void
		{
			var n : int = _processors.length;
			while(--n > -1)
			{
				_processors[n].stop();
			}
		}

		/**
		 * @param data: MidiProgramChangeVO.
		 */
		public function programChange(data : MidiProgramChangeVO) : void 
		{
			if(data.number > -1)
			{
				if(_programs[data.number])
				{
					_selectedProgramIndex = data.number;
					
					var programVO : LoopedSampleProgramVO = _programs[data.number];
					_soundVO = programVO.soundData.clone();
					_start = programVO.start;
					_end = programVO.end;
					_transpose.value = programVO.transpose;
					_tune.value = programVO.tune;
					_reverse.value = programVO.reverse;
					_loop.value = programVO.loop;
				}
				else
				{
					/** Program doesn't exist. Change nothing. */
					return;
				}
			}
			else
			{
				_selectedProgramIndex = -1;
				
				_soundVO = null;
				_start = -1;
				_end = -1;
				_transpose.value = 0;
				_tune.value = 0;
				_reverse.value = false;
				_loop.value = false;
			}
			
			facade.sendNotification(WaveEditorProxy.PROGRAM_SOUND_CHANGED, _soundVO);
			facade.sendNotification(WaveEditorProxy.PROGRAM_START_CHANGED, _start);
			facade.sendNotification(WaveEditorProxy.PROGRAM_END_CHANGED, _end);
			facade.sendNotification(WaveEditorProxy.PROGRAM_TRANSPOSE_CHANGED, _transpose.value);
			facade.sendNotification(WaveEditorProxy.PROGRAM_TUNE_CHANGED, _tune.value);
			facade.sendNotification(WaveEditorProxy.PROGRAM_REVERSE_CHANGED, _reverse.value);
			facade.sendNotification(WaveEditorProxy.PROGRAM_LOOP_CHANGED, _loop.value);
		}

		/**
		 * Adjust start position of the sample (for playback).
		 * @param startIndex : Index of start sample within audio file.
		 */
		public function set programStart(startIndex : int) : void
		{
			_start = startIndex;
			
			updateStartOnAllProcessors();
			
			facade.sendNotification(WaveEditorProxy.PROGRAM_START_CHANGED, _start);
		}

		/**
		 * Adjust end position of the sample (for playback).
		 * @param endIndex : Index of end sample within audio file.
		 */
		public function set programEnd(endIndex : int) : void
		{
			_end = endIndex;
			
			updateEndOnAllProcessors();
			
			facade.sendNotification(WaveEditorProxy.PROGRAM_END_CHANGED, _end);
		}

		/**
		 * Change transpose.
		 * @param transpose : transpose value of the sample. Usually in semitones.
		 */
		public function set programTranspose(transpose : Number) : void
		{
			_transpose.value = transpose;
			
			updateRateOnAllProcessors();
			
			facade.sendNotification(WaveEditorProxy.PROGRAM_TRANSPOSE_CHANGED, _transpose.value);
		}

		/**
		 * Change tune.
		 * @param tune : tune value of the sample. Usually up to half a semitone up or down.
		 */
		public function set programTune(tune : Number) : void
		{
			_tune.value = tune;
			
			updateRateOnAllProcessors();
			
			facade.sendNotification(WaveEditorProxy.PROGRAM_TUNE_CHANGED, _tune.value);
		}

		/**
		 * Reverses the audio vector.
		 * @param reverse : reverse on/off.
		 */
		public function set programReverse(reverse : Boolean) : void
		{
			_reverse.value = reverse;
			
			/** Stop the sound that currently plays (if there is any). */
			allNotesOff();
			
			/** The actual reverse operation. */
			_soundVO.samples.reverse();
			
			/** Adjust the start and end point. */
			var newEnd : int = _soundVO.numSamplesTotal - _start;
			programStart = _soundVO.numSamplesTotal - _end;
			programEnd = newEnd;
			
			facade.sendNotification(WaveEditorProxy.PROGRAM_REVERSE_CHANGED, _reverse.value);
		}

		/**
		 * Sets looped playback of the audio.
		 * @param loop : Switches loop on/off.
		 */
		public function set programLoop(loop : Boolean) : void
		{
			_loop.value = loop;
			
			facade.sendNotification(WaveEditorProxy.PROGRAM_LOOP_CHANGED, _loop.value);
		}

		/**
		 * @param processor : IAudioProcessor
		 */
		private function addProcessor(processor : IAudioProcessor) : void
		{
			if(!processor is LoopedSampleProcessor) return;
			
			_processors.splice(0, 0, processor);
			
			facade.sendNotification(AppFacade.PROCESSOR_ADDED, processor, getProxyName());
		}
		
		/**
		 * When transpose or tune are changed, update the playback speed of all processors.
		 */
		private function updateRateOnAllProcessors() : void
		{
			var rate : Number = Conversions.semitoneToRate(_transpose.value + _tune.value);
			var n : int = _processors.length;
			while(--n > -1)
			{
				_processors[n].rate = rate;
			}
		}
		
		/**
		 * When transpose or tune are changed, update the playback speed of all processors.
		 */
		private function updateStartOnAllProcessors() : void
		{
			var n : int = _processors.length;
			while(--n > -1)
			{
				_processors[n].start = _start;
			}
		}
		
		/**
		 * When transpose or tune are changed, update the playback speed of all processors.
		 */
		private function updateEndOnAllProcessors() : void
		{
			var n : int = _processors.length;
			while(--n > -1)
			{
				_processors[n].end = _end;
			}
		}
	}
}
