package com.hisschemoller.utils 
{
	import de.popforge.audio.output.Audio;

	import com.hisschemoller.model.AudioDriverProxy;
	import com.hisschemoller.utils.audio.vo.SampleVO;

	import flash.utils.ByteArray;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Jan 16, 2010
	 */
	public class Conversions 
	{
		/**
		 * Covert semitone shift to playback speed multiplier.
		 * For example: -12 returns 0.5, 12 returns 2, 0 returns 1.
		 */
		public static function semitoneToRate(semitone : Number) : Number
		{
			return Math.pow(2, semitone / 12);
		}
		
		/**
         * From: http://www.sengpielaudio.com/calculator-db-volt.htm
         * @param value: amplitude.
         * @return: decibel.
         */
        public static function amplitudeToDecibel(value : Number) : Number
        {
          	// return Math.round(1000000000 * 20 * (Math.log(value / 1) / Math.log(10))) / 1000000000;
          	return 20 * (Math.log(value / 1) / Math.log(10));
        }

        /**
         * From: http://www.sengpielaudio.com/calculator-db-volt.htm
         * @param value: decibel.
         * @return: amplitude.
         */
        public static function decibelToAmplitude(value : Number) : Number
        {
            return Math.exp((value / 20) * Math.log(10));
		}
		
		/**
		 * Convert a byteArray that contains audio samples to a vector of SampleVO objects.
		 * Note that the byteArray is always stereo because that's what the decompilers produce.
		 */
		public static function byteArrayToVector(byteArray : ByteArray, channels : int) : Vector.<SampleVO>
		{
			var i : int = -1;
			var numSamples : int = byteArray.length / (AudioDriverProxy.BYTES_PER_MONO_SAMPLE * Audio.STEREO);
			var samples : Vector.<SampleVO> = new Vector.<SampleVO>(numSamples, true);
			byteArray.position = 0;
			
			while(++i < numSamples)
			{
				var sample : SampleVO = new SampleVO();
				
				if(channels == Audio.MONO)
				{
					sample.left = byteArray.readFloat();
					sample.right = sample.left;
					byteArray.readFloat();
				}
				else if(channels == Audio.STEREO)
				{
					sample.left = byteArray.readFloat();
					sample.right = byteArray.readFloat();
				}
				
				samples[i] = sample;
			}
			
			return samples;
		}
	}
}
