package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.AudioEngineProxy;
	import com.hisschemoller.model.enum.FileDataType;
	import com.hisschemoller.model.vo.AlertVO;
	import com.hisschemoller.utils.audio.processors.IAudioExporter;
	import com.hisschemoller.utils.audio.processors.IAudioInstrument;
	import com.hisschemoller.utils.midi.MidiNoteOffVO;
	import com.hisschemoller.utils.midi.MidiNoteOnVO;
	import com.hisschemoller.view.components.ControlBar;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.patterns.mediator.Mediator;

	import flash.events.Event;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Jan 5, 2010
	 */
	public class ControlBarMediator extends Mediator implements IMediator 
	{
		public static const NAME : String = 'ControlBarMediator';
		private var _aboutText : String;

		public function ControlBarMediator(viewComponent : Object)
		{
			super(NAME, viewComponent); 
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_aboutText = "Audio Editor v1.1. Designed and programmed by Wouter Hisschemöller. ";
			_aboutText += "Visit <a href='http://www.hisschemoller.com'><u>www.hisschemoller.com</u></a> to download the source code.\n\n";
			_aboutText += "Code by others I used: " +
				"<a href='http://code.google.com/p/minimalcomps/'><u>MinimalComps</u></a>, " +
				"<a href='http://www.flexiblefactory.co.uk/flexible/?p=46'><u>MP3FileReferenceLoaderLib</u></a>, " +
				"<a href='http://code.google.com/p/popforge'><u>PopForge</u></a>, " +
				"<a href='http://puremvc.org'><u>PureMVC</u></a>, " +
				"<a href='http://www.nesium.com/products/trazzle/'><u>Trazzle</u></a>, " +
				"<a href='http://www.greensock.com/tweenlite/'><u>TweenLite</u></a>.";
			
			controlBar.addEventListener(ControlBar.PLAY_PRESSED, buttonEventHandler);
			controlBar.addEventListener(ControlBar.PLAY_RELEASED, buttonEventHandler);
			controlBar.addEventListener(ControlBar.OPEN_BUTTON_CLICK, buttonEventHandler);
			controlBar.addEventListener(ControlBar.SAVE_BUTTON_CLICK, buttonEventHandler);
			controlBar.addEventListener(ControlBar.ABOUT_BUTTON_CLICK, buttonEventHandler);
		}

		/**
		 * 
		 */
		private function buttonEventHandler(event : Event) : void
		{
			var instrument : IAudioInstrument = (AudioEngineProxy(facade.retrieveProxy(AudioEngineProxy.NAME))).selectedInstrument;
			
			switch(event.type)
			{
				case ControlBar.PLAY_PRESSED:
					var noteOn : MidiNoteOnVO = new MidiNoteOnVO();
					noteOn.channel = 0;
					noteOn.deltaTime = 0;
					noteOn.pitch = 60;
					noteOn.velocity = 100;
					instrument.noteOn(noteOn);
					break;
					
				case ControlBar.PLAY_RELEASED:
					var noteOff : MidiNoteOffVO = new MidiNoteOffVO();
					noteOff.channel = 0;
					noteOff.deltaTime = 0;
					noteOff.pitch = 60;
					noteOff.velocity = 0;
					instrument.noteOff(noteOff);
					break;
					
				case ControlBar.OPEN_BUTTON_CLICK:
					var xml : XML = 
						<instrument id="" type="pitchedsample">
							<programs>
								<program id="" index="" type="loopedsample" soundid="" start="" end="" reverse="false" transpose="0" tune="0" />
							</programs>
						</instrument>;
					xml.@id = instrument.id;
					facade.sendNotification(AppFacade.FILE_DATA, xml, FileDataType.OPEN_LOCAL_AUDIO_FILE);
					break;
					
				case ControlBar.SAVE_BUTTON_CLICK:
					if(instrument is IAudioExporter) (instrument as IAudioExporter).exportAudioFile();
					break;
					
				case ControlBar.ABOUT_BUTTON_CLICK:
					facade.sendNotification(AppFacade.ALERT, new AlertVO("about", _aboutText));
					break;
			}
		}

		/**
		 * 
		 */
		private function get controlBar() : ControlBar
		{
			return viewComponent as ControlBar;
		}
	}
}
