package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.WaveEditorProxy;
	import com.hisschemoller.model.vo.WaveDisplayVO;
	import com.hisschemoller.view.components.WaveDisplayControls;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;

	import flash.events.Event;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Apr 4, 2010
	 */
	public class WaveDisplayControlsMediator extends Mediator implements IMediator 
	{
		public static const NAME_SUFFIX : String = "_WaveDisplayControlsMediator";
		
		public function WaveDisplayControlsMediator(name : String, viewComponent : Object)
		{
			super(name, viewComponent);
		}

		/**
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [AppFacade.PROCESSOR_ADDED,
					AppFacade.PROCESSOR_REMOVED, 
					WaveEditorProxy.DISPLAY_POSITION_CHANGED, 
					WaveEditorProxy.DISPLAY_HORIZONTAL_ZOOM_CHANGED, 
					WaveEditorProxy.DISPLAY_VERTICAL_ZOOM_CHANGED,
					WaveEditorProxy.PROGRAM_SOUND_CHANGED, 
					WaveEditorProxy.PROGRAM_REVERSE_CHANGED];
		}

		/**
		 * 
		 */
		override public function handleNotification(notification : INotification) : void
		{
			switch(notification.getName())
			{
				case WaveEditorProxy.DISPLAY_POSITION_CHANGED:
					controls.updatePositionScrollBar(WaveDisplayVO(notification.getBody()));
					break;
				case WaveEditorProxy.DISPLAY_HORIZONTAL_ZOOM_CHANGED:
					controls.updateHZoomSlider(Number(notification.getBody()));
					break;
				case WaveEditorProxy.DISPLAY_VERTICAL_ZOOM_CHANGED:
					controls.updateVZoomSlider(Number(notification.getBody()));
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			controls.addEventListener(WaveDisplayControls.HORIZONTAL_ZOOM_CHANGE, handleHZoomSliderChange);
			controls.addEventListener(WaveDisplayControls.VERTICAL_ZOOM_CHANGE, handleVZoomSliderChange);
			controls.addEventListener(WaveDisplayControls.POSITION_CHANGE, handlePositionScrollBarChange);
		}

		/**
		 * 
		 */
		private function handleHZoomSliderChange(event : Event) : void
		{
			facade.sendNotification(WaveEditorProxy.DISPLAY_UPDATE_HORIZONTAL_ZOOM, controls.hZoomSliderValue);
		}

		/**
		 * 
		 */
		private function handleVZoomSliderChange(event : Event) : void
		{
			facade.sendNotification(WaveEditorProxy.DISPLAY_UPDATE_VERTICAL_ZOOM, controls.vZoomSliderValue);
		}

		/**
		 * 
		 */
		private function handlePositionScrollBarChange(event : Event) : void
		{
			facade.sendNotification(WaveEditorProxy.DISPLAY_UPDATE_POSITION, controls.positionScrollBarPosition);
		}

		/**
		 * 
		 */
		private function get controls() : WaveDisplayControls
		{
			return viewComponent as WaveDisplayControls;
		}
	}
}
