package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.AudioDriverProxy;
	import com.hisschemoller.model.WaveEditorProxy;
	import com.hisschemoller.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.model.vo.WaveDisplayVO;
	import com.hisschemoller.utils.audio.processors.IAudioProcessor;
	import com.hisschemoller.view.components.WaveDisplayLocators;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;

	import flash.events.Event;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Apr 5, 2010
	 */
	public class WaveDisplayLocatorsMediator extends Mediator implements IMediator 
	{
		public static const NAME_SUFFIX : String = "_WaveDisplayLocatorsMediator";
		private var _editorProxy : WaveEditorProxy;
		
		public function WaveDisplayLocatorsMediator(name : String, viewComponent : Object)
		{
			super(name, viewComponent);
		}

		/**
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [AudioDriverProxy.ENTER_FRAME,
					AppFacade.PROCESSOR_ADDED,
					AppFacade.PROCESSOR_REMOVED,
					WaveEditorProxy.DISPLAY_POSITION_CHANGED,
					WaveEditorProxy.PROGRAM_START_CHANGED,
					WaveEditorProxy.PROGRAM_END_CHANGED];
		}

		/**
		 * 
		 */
		override public function handleNotification(notification : INotification) : void
		{
			_editorProxy = facade.retrieveProxy(getMediatorName().substr(0, getMediatorName().length - NAME_SUFFIX.length)) as WaveEditorProxy;
			
			switch(notification.getName())
			{
				case AudioDriverProxy.ENTER_FRAME:
					locators.enterFrame(AudioStreamInfoVO(notification.getBody()));
					break;
				case AppFacade.PROCESSOR_ADDED:
					if(getMediatorName().indexOf(notification.getType()) != -1) locators.processorAdded(IAudioProcessor(notification.getBody()));
					break;
				case AppFacade.PROCESSOR_REMOVED:
					if(getMediatorName().indexOf(notification.getType()) != -1) locators.processorRemoved();
					break;
				case WaveEditorProxy.DISPLAY_POSITION_CHANGED:
					locators.displayUpdated(WaveDisplayVO(notification.getBody()));
					break;
				case WaveEditorProxy.PROGRAM_START_CHANGED:
					locators.startLocatorChanged(int(notification.getBody()));
					break;
				case WaveEditorProxy.PROGRAM_END_CHANGED:
					locators.endLocatorChanged(int(notification.getBody()));
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			locators.addEventListener(WaveDisplayLocators.START_PLAY_LOCATOR_CHANGE, locatorChangeHandler);
			locators.addEventListener(WaveDisplayLocators.END_PLAY_LOCATOR_CHANGE, locatorChangeHandler);
		}

		/**
		 * Called when a locator is dragged
		 * @param event : event whose type indicates which locator was dragged.
		 */
		private function locatorChangeHandler(event : Event) : void
		{
			switch(event.type)
			{
				case WaveDisplayLocators.START_PLAY_LOCATOR_CHANGE:
					_editorProxy.programStart = locators.getLocatorValue(event.type);
					break;
				case WaveDisplayLocators.END_PLAY_LOCATOR_CHANGE:
					_editorProxy.programEnd = locators.getLocatorValue(event.type);
					break;
			}
		}

		/**
		 * 
		 */
		private function get locators() : WaveDisplayLocators
		{
			return viewComponent as WaveDisplayLocators;
		}
	}
}
