package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.WaveEditorProxy;
	import com.hisschemoller.model.vo.WaveDisplayVO;
	import com.hisschemoller.utils.audio.processors.IAudioProcessor;
	import com.hisschemoller.utils.audio.vo.SoundVO;
	import com.hisschemoller.view.components.WaveDisplay;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;

	import flash.events.Event;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 19, 2010
	 */
	public class WaveDisplayMediator extends Mediator implements IMediator 
	{
		public static const NAME_SUFFIX : String = "_WaveDisplayMediator";

		public function WaveDisplayMediator(name : String, viewComponent : Object)
		{
			super(name, viewComponent);
		}

		/**
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [AppFacade.PROCESSOR_ADDED,
					AppFacade.PROCESSOR_REMOVED, 
					WaveEditorProxy.DISPLAY_UPDATE_POSITION, 
					WaveEditorProxy.DISPLAY_UPDATE_HORIZONTAL_ZOOM, 
					WaveEditorProxy.DISPLAY_UPDATE_VERTICAL_ZOOM,
					WaveEditorProxy.PROGRAM_SOUND_CHANGED, 
					WaveEditorProxy.PROGRAM_REVERSE_CHANGED];
		}

		/**
		 * 
		 */
		override public function handleNotification(notification : INotification) : void
		{
			switch(notification.getName())
			{
				case AppFacade.PROCESSOR_ADDED:
					if(getMediatorName().indexOf(notification.getType()) != -1) waveDisplay.processorAdded(IAudioProcessor(notification.getBody()));
					break;
				case AppFacade.PROCESSOR_REMOVED:
					if(getMediatorName().indexOf(notification.getType()) != -1) waveDisplay.processorRemoved();
					break;
				case WaveEditorProxy.DISPLAY_UPDATE_POSITION:
					waveDisplay.firstSampleInView = int(notification.getBody());
					break;
				case WaveEditorProxy.DISPLAY_UPDATE_HORIZONTAL_ZOOM:
					waveDisplay.horizontalZoomLevel = Number(notification.getBody());
					break;
				case WaveEditorProxy.DISPLAY_UPDATE_VERTICAL_ZOOM:
					waveDisplay.verticalZoomLevel = Number(notification.getBody());
					break;
				//case WaveDisplayProxy.GRAPHIC_UPDATED:
				//	var displayVO : WaveDisplayInfoVO = WaveDisplayInfoVO(notification.getBody());
				//	waveDisplay.updateGraphic(displayVO.bitmapData);
				//	break;
				case WaveEditorProxy.PROGRAM_SOUND_CHANGED:
					waveDisplay.updateSound(SoundVO(notification.getBody()));
					break;
				//case WaveDisplayProxy.CHANNELS_CHANGED:
				//	waveDisplay.updateChannels(uint(notification.getBody()));
				//	break;
				case WaveEditorProxy.PROGRAM_REVERSE_CHANGED:
					waveDisplay.toggleReverse();
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			waveDisplay.addEventListener(WaveDisplay.HORIZONTAL_ZOOM_CHANGE, handleDisplayHZoomLevelChange);
			waveDisplay.addEventListener(WaveDisplay.VERTICAL_ZOOM_CHANGE, handleDisplayVZoomLevelChange);
			waveDisplay.addEventListener(WaveDisplay.POSITION_CHANGE, handleDisplayPositionChange);
		}

		/**
		 * 
		 */
		private function handleDisplayHZoomLevelChange(event : Event) : void
		{
			facade.sendNotification(WaveEditorProxy.DISPLAY_HORIZONTAL_ZOOM_CHANGED, waveDisplay.horizontalZoomLevel);
		}

		/**
		 * 
		 */
		private function handleDisplayVZoomLevelChange(event : Event) : void
		{
			facade.sendNotification(WaveEditorProxy.DISPLAY_VERTICAL_ZOOM_CHANGED, waveDisplay.verticalZoomLevel);
		}

		/**
		 * 
		 */
		private function handleDisplayPositionChange(event : Event) : void
		{
			var waveDisplayVO : WaveDisplayVO = new WaveDisplayVO();
			waveDisplayVO.currentSamplesPerPixel = waveDisplay.currentSamplesPerPixel;
			waveDisplayVO.firstSampleInView = waveDisplay.firstSampleInView;
			waveDisplayVO.numSamplesInView = waveDisplay.numSamplesInView;
			waveDisplayVO.numSamplesTotal = waveDisplay.numSamplesTotal;
			
			facade.sendNotification(WaveEditorProxy.DISPLAY_POSITION_CHANGED, waveDisplayVO);
		}

		/**
		 * 
		 */
		private function get waveDisplay() : WaveDisplay
		{
			return viewComponent as WaveDisplay;
		}
	}
}
