package com.hisschemoller.view.components 
{
	import com.bit101.components.Component;
	import com.bit101.components.Slider;
	import com.greensock.TweenLite;
	import com.greensock.easing.Strong;
	import com.hisschemoller.view.components.enum.Colors;
	import com.hisschemoller.view.components.fonts.EmbeddedFont;
	import com.hisschemoller.view.components.fonts.HelveticaNeueRegular;
	import com.hisschemoller.view.components.text.FormattedTextField;
	import com.hisschemoller.view.components.ui.BasicScrollBar;
	import com.hisschemoller.view.components.ui.TextButton;

	import mx.effects.easing.Quadratic;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.AntiAliasType;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	/**
	 * @author Wouter Hisschemöller
	 * (c) Jul 19, 2009
	 */
	public class Help extends Component 
	{
		public static const CLOSE_CLICK : String = "closeClick";
		private var _closeButton : TextButton;
		private var _header : FormattedTextField;
		private var _body : FormattedTextField;
		private var _scrollBar : BasicScrollBar;

		public function Help(container : DisplayObjectContainer)
		{
			super(container);
		}

		/**
		 * 
		 */
		public function show() : void
		{
			visible = true;
			y = _height;
			_body.scrollV = 0;
			
			TweenLite.to(this, 0.3, {y:0, ease:Strong.easeInOut});
		}

		/**
		 * 
		 */
		public function hide() : void
		{
			TweenLite.to(this, 0.3, {y:_height, ease:Quadratic.easeInOut, onComplete:hideCompleteHandler});
		}
		
		/**
		 * Draw changed state.
		 */
		override public function draw():void
		{
			y = _height;
			
			graphics.clear();
			graphics.beginFill(Colors.ORANGE);
			graphics.drawRect(0, 0, _width, _height);
			
			super.draw();
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			visible = false;
			
			_closeButton = new TextButton(this, 430, 510, "close", handleCloseClick);
			
			var helpText : String = "<font size='12'>";
			helpText += "This application loads and plays MP3 files and shows their waveform graphically. On its own it's not so useful, but in a larger application where audio needs to be edited a simple waveform display like this might be exactly what is needed. This application is just a little test of how to program such a waveform display.";
			helpText += "<br><br>";
			helpText += "<font size='16'>How it works</font><br>";
			helpText += "A default MP3 file is loaded and displayed at startup. But you can load your own MP3. Click the 'load mp3' button and browse to an MP3 file on your computer. Opening a large MP3 file might take a long time however. Best to try with short files. Less than five minutes for example.";
			helpText += "<br><br>";
			helpText += "<font size='16'>Controls</font><br>";
			helpText += "<li>Once loaded the complete waveform is shown from start to end.</li>";
			helpText += "<li>Drag the 'S' and 'E' markers to select a smaller portion of the MP3.</li>";
			helpText += "<li>Click 'play mp3' to play the selected part of the MP3, or press the SPACE bar as a shortcut for the 'play mp3' button.</li>";
			helpText += "<li>Click 'reverse' to to toggle reverse playback of the MP3.</li>";
			helpText += "<li>Drag the 'transpose' slider to change the playback speed. The speed is adjusted so that the pitch is changed from one octave down to one octave up in semitone steps.</li>";
			helpText += "<li>Zoom in to a detail of the waveform with the 'zoom horizontal' slider.</li>";
			helpText += "<li>When zoomed in move through the waveform with the scrollbar directly under the waveform.</li>";
			helpText += "<li>Alternatively press and drag the mouse on the display area for finer control: Drag vertical to zoom and horizontal to move the waveform.</li>";
			helpText += "<li>Magnify the waveform vertically with the 'zoom vertical' slider. This is useful to inspect very quiet parts of the sound.</li>";
			helpText += "<br><br>";
			helpText += "<font size='16'>About the display</font><br>";
			helpText += "<li>In Flash sounds have a sample frequency (or sample rate) of 44100 samples per second. Which is the same sample rate as audio CDs and is sort of a standard sample rate.</li>";
			helpText += "<li>At the highest zoom level each sample takes one pixel of the display (horizontally).</li>";
			helpText += "<li>At lower zoom levels each pixel shows the average of several samples. The number of samples is displayed above the waveform at 'samples per pixel'.</li>";
			helpText += "<li>The first and last sample in view are shown at 'first sample' and 'last sample'.</li>";
			helpText += "<br></font>";
			
			_header = new FormattedTextField(EmbeddedFont.getTextFormat(HelveticaNeueRegular.FONT_NAME), 32, Colors.WHITE, 300, false, AntiAliasType.ADVANCED);
			_header.text = "information";
			_header.x = 7;
			_header.y = 4;
			addChild(_header);
			
			var textFormat : TextFormat = EmbeddedFont.getTextFormat(HelveticaNeueRegular.FONT_NAME);
			textFormat.leading = 4;
			
			_body = new FormattedTextField(textFormat, 12, Colors.WHITE, 300, true, AntiAliasType.ADVANCED);
			_body.x = 9;
			_body.y = 42;
			_body.width = 490;
			_body.height = 460;
			_body.autoSize = TextFieldAutoSize.NONE;
			_body.htmlText = helpText;
			_body.addEventListener(Event.SCROLL, textScrollHandler);
			addChild(_body);
			
			_scrollBar = new BasicScrollBar(Slider.VERTICAL, this, 510, 42, scrollBarChangeHandler);
			_scrollBar.setSliderParams(0, _body.maxScrollV, 0);
			_scrollBar.setThumbPercent(_body.bottomScrollV / ((_body.maxScrollV + _body.bottomScrollV) - 0));
			_scrollBar.setPageSize(10);
			_scrollBar.setSize(20, 460);
		}

		/**
		 * 
		 */
		private function scrollBarChangeHandler(event : Event) : void
		{
			_body.scrollV = _scrollBar.value;
		}

		/**
		 * 
		 */
		private function textScrollHandler(event : Event) : void
		{
			_scrollBar.value = _body.scrollV;
		}

		/**
		 * 
		 */
		private function hideCompleteHandler() : void
		{
			visible = false;
		}

		/**
		 * 
		 */
		private function handleCloseClick(event : MouseEvent) : void
		{
			hide();
		}
	}
}
