package com.hisschemoller.view.components 
{
	import de.popforge.audio.output.Audio;

	import com.bit101.components.Component;
	import com.hisschemoller.utils.audio.processors.IAudioProcessor;
	import com.hisschemoller.utils.audio.vo.SampleVO;
	import com.hisschemoller.utils.audio.vo.SoundVO;

	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 26, 2010
	 */
	public class WaveDisplay extends Component 
	{
		public static const HORIZONTAL_ZOOM_CHANGE : String = "displayHorizontalZoomChange";
		public static const VERTICAL_ZOOM_CHANGE : String = "displayVerticalZoomChange";
		public static const POSITION_CHANGE : String = "displayPositionChange";
		private var _background : Sprite;
		private var _waveform : Sprite;
		private var _soundVO : SoundVO;
		private var _sampleProcessor : IAudioProcessor;
		private var _mouseIsDown : Boolean;
		private var _firstSampleInView : int = -1;
		private var _numSamplesInView : int = -1;
		private var _numSamplesTotal : int = -1;
		private var _currentSamplesPerPixel : Number = -1;
		private var _minSamplesPerPixel : Number = 1;
		private var _maxSamplesPerPixel : Number = 1;
		private var _minVerticalZoomAmount : Number = 1;
		private var _maxVerticalZoomAmount : Number = 10;
		private var _pollingInterval : Number = 1;
		private var _numPolledSamples : int;
		private var _samplesToSkip : int;
		private var _verticalZoomAmount : Number;
		private var _zoomCenterPosition : Number;
		private var _sampleAtZoomCenter : int;
		private var _displayAmplitudeHeight : int;
		private var _xAtMouseDown : Number;
		private var _previousMouseY : Number;
		private var _filledGraphicTreshold : int = 40; //20;
		private var _displayMiddleL : int;
		private var _displayMiddleR : int;

		public function WaveDisplay(container : DisplayObjectContainer, xpos : Number = 0, ypos : Number = 0)
		{
			setSize(300, 200);
			super(container, xpos, ypos);
		}

		/**
		 * Sets the size of the component.
		 * @param w The width of the component.
		 * @param h The height of the component.
		 */
		override public function setSize(w : Number, h : Number) : void
		{
			_width = w;
			_height = h;
			
			updateSizes();
			invalidate();
		}

		/**
		 * Redraws the display.
		 */
		override public function draw() : void
		{
			/** White background. */
			_background.graphics.clear();
			_background.graphics.beginFill(0xFFFFFF);
			_background.graphics.drawRect(0, 0, _width, _height);
			
			if(!_soundVO) return;
			
			_waveform.graphics.clear();
			_waveform.graphics.lineStyle(1.5, 0x990000);
			_waveform.graphics.moveTo(0, _displayMiddleL);
			
			var drawFill : Boolean = (_currentSamplesPerPixel > _filledGraphicTreshold);
			
			switch(_soundVO.channels)
			{
				case Audio.MONO:
					if(drawFill) drawMonoAsFill();
					else drawMonoAsLine();
					break;
				case Audio.STEREO:
					if(drawFill) drawStereoAsFill();
					else drawStereoAsLine();
					break;
			}
		}

		/**
		 * Change the sound to display.
		 * @param soundVO : The sound to display.
		 */
		public function updateSound(soundVO : SoundVO) : void
		{
			if(soundVO == _soundVO) return;
			
			if(!soundVO) 
			{
				clear();
				return;
			}
			
			_soundVO = soundVO;
			_numSamplesTotal = _soundVO.numSamplesTotal;
			_firstSampleInView = 0;
			_zoomCenterPosition = 0.5;
			_sampleAtZoomCenter = _firstSampleInView + Math.round(_numSamplesTotal * _zoomCenterPosition);
			verticalZoomLevel = 0;
			
			updateSizes(true);
			invalidate();
			
			dispatchEvent(new Event(WaveDisplay.HORIZONTAL_ZOOM_CHANGE));
			dispatchEvent(new Event(WaveDisplay.VERTICAL_ZOOM_CHANGE));
			dispatchEvent(new Event(WaveDisplay.POSITION_CHANGE));
		}

		/**
		 * 
		 */
		public function toggleReverse() : void
		{
			_firstSampleInView = _soundVO.numSamplesTotal - _firstSampleInView - _numSamplesInView;
			invalidate();
			
			dispatchEvent(new Event(WaveDisplay.POSITION_CHANGE));
		}

		/**
		 * An IAudioProcessor just started. It should be tracked if it plays the selected program.
		 */
		public function processorAdded(audioProcessor : IAudioProcessor) : void
		{
			_sampleProcessor = audioProcessor;
		}

		/**
		 * An IAudioProcessor just was removed.
		 */
		public function processorRemoved() : void
		{
			_sampleProcessor = null;
		}

		/**
		 * @return : horizontal zoom as Number between 0 and 1.
		 */
		public function get horizontalZoomLevel() : Number
		{
			var value : Number = (_currentSamplesPerPixel - _minSamplesPerPixel) / (_maxSamplesPerPixel - _minSamplesPerPixel);
			var valueUnscaled : Number = Math.sqrt(value);
			
			return valueUnscaled;
		}

		/**
		 * @param value : Number between 0 and 1.
		 */
		public function set horizontalZoomLevel(value : Number) : void
		{
			if(_mouseIsDown) return;
			
			/** Scale value for more precision when zoomed in. */
			var valueScaled : Number = value * value;
			
			var samplesPerPixel : Number = _minSamplesPerPixel + ((_maxSamplesPerPixel - _minSamplesPerPixel) * valueScaled);
			
			setCurrentSamplesPerPixel(samplesPerPixel);
		}

		/**
		 * @return : vertical zoom as Number between 0 and 1.
		 */
		public function get verticalZoomLevel() : Number
		{
			var value : Number = _verticalZoomAmount / _displayMiddleL;
			value = (value - _minVerticalZoomAmount) / (_maxVerticalZoomAmount - _minVerticalZoomAmount);
			
			return value;
		}

		/**
		 * @param value : Number between 0 and 1
		 */
		public function set verticalZoomLevel(value : Number) : void
		{
			_verticalZoomAmount = _displayMiddleL * (_minVerticalZoomAmount + ((_maxVerticalZoomAmount - _minVerticalZoomAmount) * value));
			
			invalidate();
		}

		/**
		 * 
		 */
		public function get currentSamplesPerPixel() : Number
		{
			return _currentSamplesPerPixel;
		}

		/**
		 * 
		 */
		public function get firstSampleInView() : int
		{
			return _firstSampleInView;
		}

		/**
		 * Update the horizontal position.
		 */
		public function set firstSampleInView(firstSample : int) : void
		{
			if(_mouseIsDown) return;
			
			if(firstSample == _firstSampleInView) return;
			
			_firstSampleInView = firstSample;
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			_sampleAtZoomCenter = _firstSampleInView + Math.round(_numSamplesInView / 2);
			_zoomCenterPosition = 0.5;
			
			dispatchEvent(new Event(WaveDisplay.POSITION_CHANGE));
			
			invalidate();
		}

		/**
		 * 
		 */
		public function get numSamplesInView() : int
		{
			return _numSamplesInView;
		}

		/**
		 * 
		 */
		public function get numSamplesTotal() : int
		{
			return _numSamplesTotal;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			_background = new Sprite();
			_background.doubleClickEnabled = true;
			addChild(_background);
			
			_waveform = new Sprite();
			addChild(_waveform);
			
			doubleClickEnabled = true;
			addEventListener(MouseEvent.MOUSE_DOWN, mouseDownHandler);
			addEventListener(MouseEvent.DOUBLE_CLICK, mouseDoubleClickHandler);
		}

		/**
		 * Resets data and clears the display.
		 */
		private function clear() : void
		{
			_maxSamplesPerPixel = 1;
			_currentSamplesPerPixel = 1;
			_pollingInterval = 1;
			_firstSampleInView = 0;
			_numSamplesInView = 0;
			_numSamplesTotal = 0;
			_zoomCenterPosition = 0.5;
			_sampleAtZoomCenter = 0;
			_verticalZoomAmount = _displayMiddleL;
			_soundVO = null;
			_waveform.graphics.clear();
			
			invalidate();
		}

		/**
		 * 
		 */
		private function updateSizes(isNewSound : Boolean = false) : void
		{
			if(!_soundVO) return;
			
			if(_soundVO.channels == Audio.MONO)
			{
				_displayMiddleL = _height / 2;
				_displayMiddleR = _height / 2;
				_displayAmplitudeHeight = _height / 2;
			}
			else if(_soundVO.channels == Audio.STEREO)
			{
				_displayMiddleL = _height * 0.25;
				_displayMiddleR = _height * 0.75;
				_displayAmplitudeHeight = _height / 4;
			}
			
			_maxSamplesPerPixel = Number(_numSamplesTotal) / _width;
			
			var oldWidth : Number = _background.width;
			
			if(isNewSound) setCurrentSamplesPerPixel(_maxSamplesPerPixel);
			else setCurrentSamplesPerPixel((oldWidth / _width) * _currentSamplesPerPixel);
			
			_verticalZoomAmount = _displayMiddleL;
		}

		/**
		 * Repositions the displayed part of the wave by setting the first visible sample at the left edge of the display.
		 * @param firstSample : The first visible sample at the left edge of the display.
		 */
//		private function setFirstSampleInView(firstSample : int) : void
//		{
//			if(firstSample == _firstSampleInView) return;
//			
//			_firstSampleInView = firstSample;
//			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
//			
//			invalidate();
//		}

		/**
		 * Sets the zoom level of the display. Measured in how many samples one horizontal pixel of the display represents.
		 * @param samplesPerPixel : How many samples one horizontal pixel of the display represents.
		 */
		private function setCurrentSamplesPerPixel(samplesPerPixel : Number) : void
		{
			if(samplesPerPixel == _currentSamplesPerPixel) return;
			
			/** The new zoom level. */
			_currentSamplesPerPixel = Math.max(_minSamplesPerPixel, Math.min(samplesPerPixel, _maxSamplesPerPixel));
			_numSamplesInView = _currentSamplesPerPixel * _width;
			
			/** The new position. */
			_firstSampleInView = _sampleAtZoomCenter - (_numSamplesInView * _zoomCenterPosition);
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			/** Polling stuff. */
			_pollingInterval = Math.max(1, _currentSamplesPerPixel / 20);
			_numPolledSamples = int(_numSamplesInView / _pollingInterval);
			_samplesToSkip = Math.round(_pollingInterval - 1);
			
			dispatchEvent(new Event(WaveDisplay.HORIZONTAL_ZOOM_CHANGE));
			dispatchEvent(new Event(WaveDisplay.POSITION_CHANGE));
			
			invalidate();
		}

		/**
		 * 
		 */
		private function drawMonoAsFill() : void
		{
			var samplePosition : Number = 0;
			var vectorIndex : uint = 0;
			var pixelsPerPoint : uint = 2;
			var nextPixelSamplePosition : Number = _currentSamplesPerPixel * pixelsPerPoint;
			var numPoints : int = _width / pixelsPerPoint;
			var positiveAmplitude : Number = 0.0;
			var negativeAmplitude : Number = 0.0;
			var positives : Vector.<Number> = new Vector.<Number>(numPoints);
			var negatives : Vector.<Number> = new Vector.<Number>(numPoints);
			
			/** Search for amplitudes. */
			for(var i : int = 0;i < _numPolledSamples;++i)
			{
				/** Read the left channel. */
				var amplitude : Number = _soundVO.samples[int(_firstSampleInView + samplePosition)].left;
				
				/** Store only highest positive and negative amplitude. */
				if(amplitude > 0) positiveAmplitude = Math.max(positiveAmplitude, amplitude);
				else negativeAmplitude = Math.min(negativeAmplitude, amplitude);
				
				/** If all sample polling for the current pixel(s) is done store the result. */
				if(samplePosition >= nextPixelSamplePosition)
				{
					positives[vectorIndex] = positiveAmplitude;
					negatives[vectorIndex] = negativeAmplitude;
					vectorIndex ++;
					
					/** Prepare for next pixel / loop iteration. */
					nextPixelSamplePosition += _currentSamplesPerPixel * pixelsPerPoint;
					positiveAmplitude = 0;
					negativeAmplitude = 0;
				}
				
				samplePosition += _pollingInterval;
			}
			
			_waveform.graphics.beginFill(0x990000, 0.4);
			
			/** Draw positive left. */
			for(i = 0; i < numPoints; i++)
			{
				var xPos : int = i * pixelsPerPoint;
				var yPos : Number = _displayMiddleL + Math.min((positives[i] * _verticalZoomAmount), _displayAmplitudeHeight);
				_waveform.graphics.lineTo(xPos, yPos);
			}
			
			/** Draw negative left. */
			for(i = numPoints - 1; i >= 0; i--)
			{
				xPos = i * pixelsPerPoint;
				yPos = _displayMiddleL + Math.max((negatives[i] * _verticalZoomAmount), -_displayAmplitudeHeight);
				_waveform.graphics.lineTo(xPos, yPos);
			}
		}

		/**
		 * 
		 */
		private function drawMonoAsLine() : void
		{
			var samplePosition : Number = 0;
			var nextPixelSamplePosition : Number = _currentSamplesPerPixel;
			var pixelIndex : uint = 0;
			var positiveWaveHeight : Number = 0.0;
			var negativeWaveHeight : Number = 0.0;
			
			for(var i : int = 0;i < _numPolledSamples;++i)
			{
				/** Read the left channel. */
				var amplitude : Number = _soundVO.samples[int(_firstSampleInView + samplePosition)].left;
				
				/** Store only highest positive and negative amplitude. */
				if(amplitude > 0) positiveWaveHeight = Math.max(positiveWaveHeight, amplitude);
				else negativeWaveHeight = Math.min(negativeWaveHeight, amplitude);
				
				/** If all sample polling for the current pixel is done draw the result. */
				if(samplePosition >= nextPixelSamplePosition)
				{
					/** Zoomed in: Draw waveform as line. */
					if(positiveWaveHeight > (negativeWaveHeight * -1))
					{
						_waveform.graphics.lineTo(pixelIndex, _displayMiddleL + Math.min((positiveWaveHeight * _verticalZoomAmount), _displayAmplitudeHeight));
					}
					else
					{
						_waveform.graphics.lineTo(pixelIndex, _displayMiddleL + Math.max((negativeWaveHeight * _verticalZoomAmount), -_displayAmplitudeHeight));
					}
					
					/** Prepare for next pixel / loop iteration. */
					nextPixelSamplePosition += _currentSamplesPerPixel;
					positiveWaveHeight = 0;
					negativeWaveHeight = 0;
					pixelIndex++;
				}
				
				samplePosition += _pollingInterval;
			}
		}

		/**
		 * 
		 */
		private function drawStereoAsFill() : void
		{
			var samplePosition : Number = 0;
			var vectorIndex : uint = 0;
			var pixelsPerPoint : uint = 2;
			var nextPixelSamplePosition : Number = _currentSamplesPerPixel * pixelsPerPoint;
			var numPoints : int = _width / pixelsPerPoint;
			var positiveAmplitudeL : Number = 0.0;
			var negativeAmplitudeL : Number = 0.0;
			var positiveAmplitudeR : Number = 0.0;
			var negativeAmplitudeR : Number = 0.0;
			var positiveL : Vector.<Number> = new Vector.<Number>(numPoints);
			var negativeL : Vector.<Number> = new Vector.<Number>(numPoints);
			var positiveR : Vector.<Number> = new Vector.<Number>(numPoints);
			var negativeR : Vector.<Number> = new Vector.<Number>(numPoints);
			
			/** Search for amplitudes. */
			for(var i : int = 0;i < _numPolledSamples;++i)
			{
				/** Read the channels. */
				var sample : SampleVO = _soundVO.samples[int(_firstSampleInView + samplePosition)];
				var amplitudeL : Number = sample.left;
				var amplitudeR : Number = sample.right;
				
				/** Store only highest positive and negative amplitude. */
				if(amplitudeL > 0) positiveAmplitudeL = Math.max(positiveAmplitudeL, amplitudeL);
				else negativeAmplitudeL = Math.min(negativeAmplitudeL, amplitudeL);
				if(amplitudeR > 0) positiveAmplitudeR = Math.max(positiveAmplitudeR, amplitudeR);
				else negativeAmplitudeR = Math.min(negativeAmplitudeR, amplitudeR);
				
				/** If all sample polling for the current pixel(s) is done store the result. */
				if(samplePosition >= nextPixelSamplePosition)
				{
					positiveL[vectorIndex] = positiveAmplitudeL;
					negativeL[vectorIndex] = negativeAmplitudeL;
					positiveR[vectorIndex] = positiveAmplitudeR;
					negativeR[vectorIndex] = negativeAmplitudeR;
					vectorIndex ++;
					
					/** Prepare for next pixel / loop iteration. */
					nextPixelSamplePosition += _currentSamplesPerPixel * pixelsPerPoint;
					positiveAmplitudeL = 0;
					negativeAmplitudeL = 0;
					positiveAmplitudeR = 0;
					negativeAmplitudeR = 0;
				}
				
				samplePosition += _pollingInterval;
			}
			
			_waveform.graphics.beginFill(0x990000, 0.4);
			
			/** Draw positive left. */
			for(i = 0; i < numPoints; i++)
			{
				var xPos : int = i * pixelsPerPoint;
				var yPos : Number = _displayMiddleL + Math.min((positiveL[i] * _verticalZoomAmount), _displayAmplitudeHeight);
				_waveform.graphics.lineTo(xPos, yPos);
			}
			
			/** Draw negative left. */
			for(i = numPoints - 1; i >= 0; i--)
			{
				xPos = i * pixelsPerPoint;
				yPos = _displayMiddleL + Math.max((negativeL[i] * _verticalZoomAmount), -_displayAmplitudeHeight);
				_waveform.graphics.lineTo(xPos, yPos);
			}
			
			/** Draw positive right. */
			for(i = 0; i < numPoints; i++)
			{
				xPos = i * pixelsPerPoint;
				yPos = _displayMiddleR + Math.min((positiveR[i] * _verticalZoomAmount), _displayAmplitudeHeight);
				_waveform.graphics.lineTo(xPos, yPos);
			}
			
			/** Draw negative right. */
			for(i = numPoints - 1; i >= 0; i--)
			{
				xPos = i * pixelsPerPoint;
				yPos = _displayMiddleR + Math.max((negativeR[i] * _verticalZoomAmount), -_displayAmplitudeHeight);
				_waveform.graphics.lineTo(xPos, yPos);
			}
		}

		/**
		 * 
		 */
		private function drawStereoAsLine() : void
		{
			var nextPixelSamplePosition : Number = _currentSamplesPerPixel;
			var samplePosition : Number = 0;
			var pixelIndex : uint = 0;
			var positiveWaveHeightL : Number = 0.0;
			var negativeWaveHeightL : Number = 0.0;
			var positiveWaveHeightR : Number = 0.0;
			var negativeWaveHeightR : Number = 0.0;
			var leftX : Number = 0;
			var leftY : Number = _displayMiddleL;
			var rightX : Number = 0;
			var rightY : Number = _displayMiddleR;
			
			for(var i : int = 0;i < _numPolledSamples;++i)
			{
				/** Read the channels. */
				var sample : SampleVO = _soundVO.samples[int(_firstSampleInView + samplePosition)];
				var amplitudeL : Number = sample.left;
				var amplitudeR : Number = sample.right;
				
				/** Store only highest positive and negative amplitude. */
				if(amplitudeL > 0) positiveWaveHeightL = Math.max(positiveWaveHeightL, amplitudeL);
				else negativeWaveHeightL = Math.min(negativeWaveHeightL, amplitudeL);
				if(amplitudeR > 0) positiveWaveHeightR = Math.max(positiveWaveHeightR, amplitudeR);
				else negativeWaveHeightR = Math.min(negativeWaveHeightR, amplitudeR);
				
				/** If all sample polling for the current pixel is done draw the result. */
				if(samplePosition >= nextPixelSamplePosition)
				{
					/** Zoomed in: Draw waveform as line. */
					if(positiveWaveHeightL > (negativeWaveHeightL * -1))
					{
						_waveform.graphics.moveTo(leftX, leftY);
						leftX = pixelIndex;
						leftY = _displayMiddleL + Math.min((positiveWaveHeightL * _verticalZoomAmount), _displayAmplitudeHeight);
						_waveform.graphics.lineTo(leftX, leftY);
						
						_waveform.graphics.moveTo(rightX, rightY);
						rightX = pixelIndex;
						rightY = _displayMiddleR + Math.min((positiveWaveHeightR * _verticalZoomAmount), _displayAmplitudeHeight);
						_waveform.graphics.lineTo(rightX, rightY);
					}
					else
					{
						_waveform.graphics.moveTo(leftX, leftY);
						leftX = pixelIndex;
						leftY = _displayMiddleL + Math.max((negativeWaveHeightL * _verticalZoomAmount), -_displayAmplitudeHeight);
						_waveform.graphics.lineTo(leftX, leftY);
						
						_waveform.graphics.moveTo(rightX, rightY);
						rightX = pixelIndex;
						rightY = _displayMiddleR + Math.max((negativeWaveHeightR * _verticalZoomAmount), -_displayAmplitudeHeight);
						_waveform.graphics.lineTo(rightX, rightY);
					}
					
					pixelIndex++;
					
					/** Prepare for next pixel / loop iteration. */
					nextPixelSamplePosition += _currentSamplesPerPixel;
					positiveWaveHeightL = 0;
					negativeWaveHeightL = 0;
					positiveWaveHeightR = 0;
					negativeWaveHeightR = 0;
				}
				
				samplePosition += _pollingInterval;
			}
		}

		/**
		 * 
		 */
		private function mouseDownHandler(event : MouseEvent) : void
		{
			_mouseIsDown = true;
			_xAtMouseDown = _waveform.mouseX;
			_previousMouseY = _waveform.mouseY;
			_zoomCenterPosition = _waveform.mouseX / _width;
			_sampleAtZoomCenter = _firstSampleInView + Math.round(_numSamplesInView * _zoomCenterPosition);
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMoveHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, mouseUpHandler);
			stage.addEventListener(Event.MOUSE_LEAVE, mouseUpHandler);
		}

		/**
		 * 
		 */
		private function mouseMoveHandler(event : MouseEvent) : void
		{
			if(!_mouseIsDown) return;
			
			/** The distance the mouse was dragged. */
			var distanceX : Number = _waveform.mouseX - _xAtMouseDown;
			var distanceY : Number = _waveform.mouseY - _previousMouseY;
			_previousMouseY = _waveform.mouseY;
			
			/** The new zoom level. */
			_currentSamplesPerPixel -= (distanceY * _currentSamplesPerPixel * 0.01);
			_currentSamplesPerPixel = Math.max(_minSamplesPerPixel, Math.min(_currentSamplesPerPixel, _maxSamplesPerPixel));
			_numSamplesInView = _currentSamplesPerPixel * _width;
			
			/** The new position. */
			_zoomCenterPosition = (_xAtMouseDown + distanceX) / _width;
			_firstSampleInView = _sampleAtZoomCenter - (_numSamplesInView * _zoomCenterPosition);
			_firstSampleInView = Math.max(0, Math.min(_firstSampleInView, _numSamplesTotal - _numSamplesInView));
			
			/** Polling stuff. */
			_pollingInterval = Math.max(1, _currentSamplesPerPixel / 20);
			_numPolledSamples = int(_numSamplesInView / _pollingInterval);
			_samplesToSkip = Math.round(_pollingInterval - 1);
			
			if(_mouseIsDown) dispatchEvent(new Event(WaveDisplay.HORIZONTAL_ZOOM_CHANGE));
			if(_mouseIsDown) dispatchEvent(new Event(WaveDisplay.POSITION_CHANGE));
			
			invalidate();
		}

		/**
		 * 
		 */
		private function mouseUpHandler(event : Event) : void
		{
			_mouseIsDown = false;
			
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, mouseMoveHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, mouseUpHandler);
			stage.removeEventListener(Event.MOUSE_LEAVE, mouseUpHandler);
		}

		/**
		 * 
		 */
		private function mouseDoubleClickHandler(event : MouseEvent) : void
		{
			log("doubleclick");
			dispatchEvent(event);
		}
	}
}
