package com.hisschemoller.view.components 
{
	import assets.PlayBar;

	import com.bit101.components.Component;
	import com.bit101.components.Slider;
	import com.hisschemoller.model.vo.WaveDisplayVO;
	import com.hisschemoller.view.components.enum.Colors;
	import com.hisschemoller.view.components.text.BodyText;
	import com.hisschemoller.view.components.ui.BasicScrollBar;
	import com.hisschemoller.view.components.ui.BasicSlider;

	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormatAlign;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Apr 3, 2010
	 */
	public class WaveDisplayControls extends Component 
	{
		public static const HORIZONTAL_ZOOM_CHANGE : String = "controlsHorizontalZoomChange";
		public static const VERTICAL_ZOOM_CHANGE : String = "controlsVerticalZoomChange";
		public static const POSITION_CHANGE : String = "controlsPositionChange";
		private var _playBar : PlayBar;
		private var _hZoomSlider : BasicSlider;
		private var _vZoomSlider : BasicSlider;
		private var _hZoomLabel : BodyText;
		private var _vZoomLabel : BodyText;
		private var _firstLabel : BodyText;
		private var _lastLabel : BodyText;
		private var _samplesPerPixelLabel : BodyText;
		private var _positionScrollBar : BasicScrollBar;
		private var _bottom : Sprite;
		private var _waveDisplayVO : WaveDisplayVO;
		
		public function WaveDisplayControls(parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0)
		{
			super(parent, xpos, ypos);
		}

		/**
		 * Sets the size of the component.
		 * @param w The width of the component.
		 * @param h The height of the component.
		 */
		override public function setSize(w : Number, h : Number) : void
		{
			_width = w;
			_height = h;
			
			invalidate();
		}

		/**
		 * Redraws the display.
		 */
		override public function draw() : void
		{
			_bottom.y = _height;
			_hZoomSlider.x = _width - 400;
			_vZoomSlider.x = _width - _vZoomSlider.width;
			_positionScrollBar.setSize(_width, 19);
			_hZoomLabel.x = _hZoomSlider.x - _hZoomLabel.width - 5;
			_vZoomLabel.x = _vZoomSlider.x - _vZoomLabel.width - 5;
		}

		/**
		 * 
		 */
		public function updatePositionScrollBar(waveDisplayVO : WaveDisplayVO) : void
		{
			_waveDisplayVO = waveDisplayVO;
			
			var min : Number = 0;
			var max : Number = waveDisplayVO.numSamplesTotal - waveDisplayVO.numSamplesInView;
			var value : Number = waveDisplayVO.firstSampleInView;
			_positionScrollBar.setSliderParams(min, max, value);
			_positionScrollBar.setThumbPercent(waveDisplayVO.numSamplesInView / waveDisplayVO.numSamplesTotal);
			_positionScrollBar.setPageSize(waveDisplayVO.numSamplesInView);
			
			
			
			//_firstLabel.text = firstSampleInView.toString();
			//_lastLabel.text = (firstSampleInView + numSamplesInView).toString();
			//_samplesPerPixelLabel.text = currentSamplesPerPixel.toFixed(2);
		}

		/**
		 * 
		 */
		public function updateHZoomSlider(value : Number) : void
		{
			_hZoomSlider.value = value;
		}

		/**
		 * 
		 */
		public function updateVZoomSlider(value : Number) : void
		{
			_vZoomSlider.value = value;
		}
		
		/**
		 * 
		 */
		public function get waveDisplayVO() : WaveDisplayVO
		{
			return _waveDisplayVO;
		}

		/**
		 * 
		 */
		public function get hZoomSliderValue() : Number
		{
			return _hZoomSlider.value;
		}

		/**
		 * 
		 */
		public function get vZoomSliderValue() : Number
		{
			return _vZoomSlider.value;
		}

		/**
		 * 
		 */
		public function get positionScrollBarPosition() : Number
		{
			return _positionScrollBar.value;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			_bottom = new Sprite();
			_bottom.y = 280;
			addChild(_bottom);
			
			_playBar = new PlayBar();
			_playBar.visible = false;
			_playBar.x = 0;
			_playBar.y = 0;
			addChild(_playBar);
			
			_hZoomSlider = new BasicSlider(Slider.HORIZONTAL, _bottom, 100, 27, handleHZoomSliderChange);
			_hZoomSlider.setSliderParams(0, 1, 0);
			_hZoomSlider.setSize(150, 11);
			
			_vZoomSlider = new BasicSlider(Slider.HORIZONTAL, _bottom, 350, 27, handleVZoomSliderChange);
			_vZoomSlider.setSliderParams(0, 1, 0);
			_vZoomSlider.setSize(150, 11);
			
			_hZoomLabel = new BodyText(13, Colors.WHITE, 100);
			_hZoomLabel.autoSize = TextFieldAutoSize.RIGHT;
			_hZoomLabel.text = "zoom horizontal";
			_hZoomLabel.y = 23;
			_bottom.addChild(_hZoomLabel);
			
			_vZoomLabel = new BodyText(13, Colors.WHITE, 100);
			_vZoomLabel.autoSize = TextFieldAutoSize.RIGHT;
			_vZoomLabel.text = "zoom vertical";
			_vZoomLabel.y = 23;
			_bottom.addChild(_vZoomLabel);
			
			_firstLabel = new BodyText(11, Colors.RUBY_RED, 200);
			_firstLabel.x = 5;
			_firstLabel.y = 437;
			addChild(_firstLabel);
			
			_lastLabel = new BodyText(11, Colors.RUBY_RED, 200);
			_lastLabel.align = TextFormatAlign.RIGHT;
			_lastLabel.autoSize = TextFieldAutoSize.RIGHT;
			_lastLabel.x = 492;
			_lastLabel.y = 437;
			addChild(_lastLabel);
			
			_samplesPerPixelLabel = new BodyText(11, Colors.RUBY_RED, 200);
			_samplesPerPixelLabel.align = TextFormatAlign.CENTER;
			_samplesPerPixelLabel.autoSize = TextFieldAutoSize.CENTER;
			_samplesPerPixelLabel.x = 250;
			_samplesPerPixelLabel.y = 437;
			addChild(_samplesPerPixelLabel);
			
			_positionScrollBar = new BasicScrollBar(Slider.HORIZONTAL, _bottom, 0, 3, handlePositionScrollBarChange);
			_positionScrollBar.setSize(500, 19);
		}

		/**
		 * 
		 */
		private function handleHZoomSliderChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplayControls.HORIZONTAL_ZOOM_CHANGE));
		}

		/**
		 * 
		 */
		private function handleVZoomSliderChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplayControls.VERTICAL_ZOOM_CHANGE));
		}

		/**
		 * 
		 */
		private function handlePositionScrollBarChange(event : Event) : void
		{
			dispatchEvent(new Event(WaveDisplayControls.POSITION_CHANGE));
		}
	}
}
