package com.hisschemoller.view.components 
{
	import com.bit101.components.Component;
	import com.hisschemoller.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.model.vo.WaveDisplayVO;
	import com.hisschemoller.utils.audio.processors.IAudioProcessor;
	import com.hisschemoller.view.components.ui.BasicLocatorBar;
	import com.hisschemoller.view.components.ui.PlayLocatorBar;
	import com.hisschemoller.view.components.ui.SelectionLocatorBar;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Apr 5, 2010
	 */
	public class WaveDisplayLocators extends Component 
	{
		public static const START_PLAY_LOCATOR_CHANGE : String = "startPlayLocatorChange";
		public static const END_PLAY_LOCATOR_CHANGE : String = "endPlayLocatorChange";
		public static const PLAY_LOCATOR_CHANGE : String = "playLocatorChange";
		private var _selectionLocatorBar : SelectionLocatorBar;
		private var _playLocatorBar : PlayLocatorBar;
		private var _sampleProcessor : IAudioProcessor;
		private var _startSample : int = -1;
		private var _endSample : int = -1;
		private var _firstSampleInView : int = -1;
		private var _numSamplesInView : int = -1;
		private var _latencyInMS : Number;
		private var _sampleIndex : int;

		public function WaveDisplayLocators(parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0)
		{
			super(parent, xpos, ypos);
		}

		/**
		 * Sets the size of the component.
		 * @param w The width of the component.
		 * @param h The height of the component.
		 */
		override public function setSize(w : Number, h : Number) : void
		{
			_selectionLocatorBar.setSize(w, h);
			_playLocatorBar.setSize(w, h);
			super.setSize(w, h);
		}

		/**
		 * Update the play bar every frame.
		 */
		public function enterFrame(streamInfo : AudioStreamInfoVO) : void
		{
			_latencyInMS = streamInfo.latencyInMS;
		}

		/**
		 * An IAudioProcessor just started. It should be tracked if it plays the selected program.
		 */
		public function processorAdded(audioProcessor : IAudioProcessor) : void
		{
			_sampleProcessor = audioProcessor;
			
			addEventListener(Event.ENTER_FRAME, enterFrameHandler);
		}

		/**
		 * An IAudioProcessor just was removed. It should be tracked if it played the selected program.
		 */
		public function processorRemoved() : void
		{
			_sampleProcessor = null;
			
			removeEventListener(Event.ENTER_FRAME, enterFrameHandler);
		}

		/**
		 * 
		 */
		public function displayUpdated(waveDisplayVO : WaveDisplayVO) : void
		{
			_firstSampleInView = waveDisplayVO.firstSampleInView;
			_numSamplesInView = waveDisplayVO.numSamplesInView;
			
			updateLocators();
		}

		/**
		 * 
		 */
		public function startLocatorChanged(startSample : int) : void
		{
			_startSample = startSample;
			
			updateLocators();
		}

		/**
		 * 
		 */
		public function endLocatorChanged(endSample : int) : void
		{
			_endSample = endSample;
			
			updateLocators();
		}

		/**
		 * Return locator position
		 */
		public function getLocatorValue(locator : String) : Number
		{
			switch(locator)
			{
				case START_PLAY_LOCATOR_CHANGE:
					return _firstSampleInView + Math.round(_numSamplesInView * _selectionLocatorBar.startValue);
					break;
				case END_PLAY_LOCATOR_CHANGE:
					return _firstSampleInView + Math.round(_numSamplesInView * _selectionLocatorBar.endValue);
					break;
			}
			
			return NaN;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			_selectionLocatorBar = new SelectionLocatorBar(this, 10, 154);
			_selectionLocatorBar.addEventListener(BasicLocatorBar.START_LOCATOR_CHANGE, playLocatorEventHandler);
			_selectionLocatorBar.addEventListener(BasicLocatorBar.END_LOCATOR_CHANGE, playLocatorEventHandler);
			
			_playLocatorBar = new PlayLocatorBar(this, 10, 154);
			_selectionLocatorBar.addEventListener(PlayLocatorBar.PLAY_LOCATOR_CHANGE, playLocatorEventHandler);
		}

		/**
		 * Update locator position.
		 */
		private function updateLocators() : void
		{
			if(_startSample > -1 && _endSample > -1)
			{
				_selectionLocatorBar.startValue = (_startSample - _firstSampleInView) / _numSamplesInView;
				_selectionLocatorBar.endValue = (_endSample - _firstSampleInView) / _numSamplesInView;
				_playLocatorBar.value = (_sampleIndex - _firstSampleInView) / _numSamplesInView;
				
				// _selectionLabel.text = "selection start: " + _startSample + "  end: " + _endSample;
			}
		}
		
		/**
		 * 
		 */
		private function updatePlayBar(sampleIndex : int) : void
		{
			_sampleIndex = sampleIndex;
			_playLocatorBar.value = (sampleIndex - _firstSampleInView) / _numSamplesInView;
		}

		/**
		 * 
		 */
		private function enterFrameHandler(event : Event) : void
		{
			new Delayer(updatePlayBar, _latencyInMS, _sampleProcessor.sampleIndex);
		}

		/**
		 * 
		 */
		private function playLocatorEventHandler(event : Event) : void
		{
			switch(event.type)
			{
				case BasicLocatorBar.START_LOCATOR_CHANGE:
					dispatchEvent(new Event(WaveDisplayLocators.START_PLAY_LOCATOR_CHANGE));
					break;
				case BasicLocatorBar.END_LOCATOR_CHANGE:
					dispatchEvent(new Event(WaveDisplayLocators.END_PLAY_LOCATOR_CHANGE));
					break;
				case PlayLocatorBar.PLAY_LOCATOR_CHANGE:
					dispatchEvent(new Event(WaveDisplayLocators.PLAY_LOCATOR_CHANGE));
					break;
			}
		}
	}
}

import flash.events.TimerEvent;
import flash.utils.Timer;

internal class Delayer
{
	private var _closure : Function;
	private var _sampleIndex : int;
	private var _timer : Timer;
	
	public function Delayer(closure : Function, delay : int, sampleIndex : int)
	{
		_closure = closure;
		_sampleIndex = sampleIndex;
		_timer = new Timer(delay, 1);
		_timer.addEventListener(TimerEvent.TIMER_COMPLETE, timerCompleteHandler);
		_timer.start();
	}

	private function timerCompleteHandler(event : TimerEvent) : void 
	{
		_timer.removeEventListener(TimerEvent.TIMER_COMPLETE, timerCompleteHandler);
		_timer = null;
		
		_closure(_sampleIndex);
	}
}
