package com.hisschemoller.view.components 
{
	import assets.ActivityLED;
	import assets.SampleEditorAsset;

	import com.bit101.components.Component;
	import com.bit101.components.Slider;
	import com.hisschemoller.view.components.enum.Colors;
	import com.hisschemoller.view.components.text.BodyText;
	import com.hisschemoller.view.components.ui.BasicSlider;
	import com.hisschemoller.view.components.ui.TextButton;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormatAlign;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Feb 19, 2010
	 */
	public class WaveEditor extends Component 
	{
		public static const REVERSE_CHANGE : String = "reverseChange";
		public static const LOOP_CHANGE : String = "loopChange";
		public static const TRANSPOSE_CHANGE : String = "transposeChange";
		public static const TUNE_CHANGE : String = "tuneChange";
		private var _waveDisplay : WaveDisplay;
		private var _waveDisplayOverlay : WaveDisplayOverlay;
		private var _waveDisplayControls : WaveDisplayControls;
		private var _waveDisplayLocators : WaveDisplayLocators;
		private var _reverseButton : TextButton;
		private var _loopButton : TextButton;
		private var _selectionLabel : BodyText;
		private var _transposeSlider : BasicSlider;
		private var _transposeLabel : BodyText;
		private var _tuneSlider : BasicSlider;
		private var _tuneLabel : BodyText;
		private var _activityLED : ActivityLED;
		
		public function WaveEditor(parent : DisplayObjectContainer)
		{
			super(parent);
		}
		
		/**
		 * 
		 */
		override public function draw() : void
		{
			var w : Number = stage.stageWidth - 40;
			var h : Number = stage.stageHeight - _waveDisplay.y - 87;
			
			_waveDisplay.setSize(w, h);
			_waveDisplayOverlay.setSize(w, h);
			_waveDisplayControls.setSize(w, h);
			_waveDisplayLocators.setSize(w, h + 19);
			
			super.draw();
		}

		/**
		 * 
		 */
		public function updatePlaybackState(isPlaying : Boolean) : void
		{
			if(isPlaying) _activityLED.gotoAndPlay("play");
			else _activityLED.gotoAndPlay("neutral");
		}
		
		/**
		 * Reference to WaveDisplay required by WaveDisplayMediator.
		 */
		public function get waveDisplay() : WaveDisplay
		{
			return _waveDisplay;
		}
		
		/**
		 * Reference to WaveDisplayOverlay required by WaveDisplayOverlayMediator.
		 */
		public function get waveDisplayOverlay() : WaveDisplayOverlay
		{
			return _waveDisplayOverlay;
		}
		
		/**
		 * Reference to WaveDisplayControls required by WaveDisplayControlsMediator.
		 */
		public function get waveDisplayControls() : WaveDisplayControls
		{
			return _waveDisplayControls;
		}
		
		/**
		 * Reference to waveDisplayLocators required by WaveDisplayLocatorsMediator.
		 */
		public function get waveDisplayLocators() : WaveDisplayLocators
		{
			return _waveDisplayLocators;
		}
		
		/**
		 * 
		 */
		public function get transpose() : Number
		{
			return _transposeSlider.value;
		}
		
		/**
		 * 
		 */
		public function set transpose(value : Number) : void
		{
			_transposeSlider.value = value;
			_transposeLabel.text = value.toFixed(2);
		}
		
		/**
		 * 
		 */
		public function get tune() : Number
		{
			return _tuneSlider.value;
		}
		
		/**
		 * 
		 */
		public function set tune(value : Number) : void
		{
			_tuneSlider.value = value;
			_tuneLabel.text = value.toFixed(2);
		}

		/**
		 * 
		 */
		public function get reverse() : Boolean
		{
			return _reverseButton.selected;
		}

		/**
		 * 
		 */
		public function set reverse(value : Boolean) : void
		{
			_reverseButton.selected = value;
		}

		/**
		 * 
		 */
		public function get loop() : Boolean
		{
			return _loopButton.selected;
		}

		/**
		 * 
		 */
		public function set loop(value : Boolean) : void
		{
			_loopButton.selected = value;
		}

		/**
		 * 
		 */
		override protected function addChildren() : void
		{
			var asset : SampleEditorAsset = new SampleEditorAsset();
			addChild(asset);
			
			_waveDisplay = new WaveDisplay(this, 10, 173);
			
			_waveDisplayOverlay = new WaveDisplayOverlay(this, 10, 173);
			
			_waveDisplayControls = new WaveDisplayControls(this, 10, 173);
			
			_waveDisplayLocators = new WaveDisplayLocators(this, 0, 0);
			
			_selectionLabel = new BodyText(11, Colors.WHITE, 200);
			_selectionLabel.autoSize = TextFieldAutoSize.LEFT;
			_selectionLabel.x = 257;
			_selectionLabel.y = 132;
			asset.addChild(_selectionLabel);
			
			_reverseButton = new TextButton(asset, 260, 85, "reverse", reverseClickHandler);
			_reverseButton.toggle = true;
			
			_loopButton = new TextButton(asset, 260, 112, "loop", loopClickHandler);
			_loopButton.toggle = true;
			
			_transposeSlider = new BasicSlider(Slider.HORIZONTAL, asset, 10, 116, transposeSliderChangeHandler);
			_transposeSlider.setSliderParams(-12, 12, 0);
			_transposeSlider.setSize(210, 16);
			_transposeSlider.tick = 1;
			
			_transposeLabel = new BodyText(11, Colors.WHITE, 85);
			_transposeLabel.align = TextFormatAlign.RIGHT;
			_transposeLabel.autoSize = TextFieldAutoSize.RIGHT;
			_transposeLabel.x = 218;
			_transposeLabel.y = 132;
			asset.addChild(_transposeLabel);
			
			_tuneSlider = new BasicSlider(Slider.HORIZONTAL, asset, 10, 85, tuneSliderChangeHandler);
			_tuneSlider.setSliderParams(-0.5, 0.5, 0);
			_tuneSlider.setSize(210, 10);
			
			_tuneLabel = new BodyText(11, Colors.WHITE, 85);
			_tuneLabel.align = TextFormatAlign.RIGHT;
			_tuneLabel.autoSize = TextFieldAutoSize.RIGHT;
			_tuneLabel.x = 218;
			_tuneLabel.y = 97;
			asset.addChild(_tuneLabel);
			
			_activityLED = asset.tActivityLED;
			
			stage.addEventListener(Event.RESIZE, stageResizeHandler);
		}

		/**
		 * 
		 */
		private function reverseClickHandler(event : MouseEvent) : void
		{
			dispatchEvent(new Event(WaveEditor.REVERSE_CHANGE));
		}

		/**
		 * 
		 */
		private function loopClickHandler(event : MouseEvent) : void
		{
			dispatchEvent(new Event(WaveEditor.LOOP_CHANGE));
		}

		/**
		 * 
		 */
		private function transposeSliderChangeHandler(event : Event) : void
		{
			dispatchEvent(new Event(WaveEditor.TRANSPOSE_CHANGE));
		}

		/**
		 * 
		 */
		private function tuneSliderChangeHandler(event : Event) : void
		{
			dispatchEvent(new Event(WaveEditor.TUNE_CHANGE));
		}
		
		/**
		 * 
		 */
		private function stageResizeHandler(event : Event) : void
		{
			invalidate();
		}
	}
}
