package com.hisschemoller.view.components.ui 
{
	import com.bit101.components.Component;

	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 22, 2010
	 */
	public class BasicLocatorBar extends Component 
	{
		public static const START_LOCATOR_CHANGE : String = "startLocatorChange";
		public static const END_LOCATOR_CHANGE : String = "endLocatorChange";
		protected var _track : Sprite;
		protected var _fill : Sprite;
		protected var _fillMask : Sprite;
		protected var _startLocator : BasicLocator;
		protected var _endLocator : BasicLocator;
		protected var _activeLocator : BasicLocator;
		protected var _minimum : Number = 0;
		protected var _maximum : Number = 1;
		protected var _startValue : Number = 0.25;
		protected var _endValue : Number = 0.75;
		protected var _isDragging : Boolean;
		
		public function BasicLocatorBar(parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0)
		{
			super(parent, xpos, ypos);
		}
		
		/**
		 * 
		 */
		override public function draw() : void
		{
			_track.width = _width;
			
			if(_startValue > _endValue) _startValue = _endValue;
			if(_endValue < _startValue) _endValue = _startValue;
			
			if(!_isDragging)
			{
				_startLocator.x = _track.x + (_startValue * _width);
				_endLocator.x = _track.x + (_endValue * _width);
			}
			
			_fillMask.x = Math.max(_minimum, _startLocator.x);
			_fillMask.width = _endLocator.x - _fillMask.x;
			
			if((_startValue < _minimum) || (_startValue > _maximum)) _startLocator.visible = false;
			else _startLocator.visible = true;
			
			if((_endValue < _minimum) || (_endValue > _maximum)) _endLocator.visible = false;
			else _endLocator.visible = true;
			
			if((_endValue < _minimum) || (_startValue > _maximum)) _fill.visible = false;
			else _fill.visible = true;
		}
		
		/**
		 * 
		 */
		public function get startValue() : Number
		{
			return _startValue;
		}

		/**
		 * @param inValue : new value for the start locator
		 */
		public function set startValue(value : Number) : void
		{
			if(_isDragging) return;
			
			_startValue = value;
			
			invalidate();
		}
		
		/**
		 * 
		 */
		public function get endValue() : Number
		{
			return _endValue;
		}
		
		/**
		 * @param inValue : new value for the end locator
		 */
		public function set endValue(inValue : Number) : void
		{
			if(_isDragging) return;
			
			_endValue = inValue;
			
			invalidate();
		}

		/**
		 * Creates and adds the child display objects of this component.
		 */
		override protected function addChildren() : void
		{
			_startLocator.addEventListener(MouseEvent.MOUSE_DOWN, locatorMouseDownHandler);
			
			_endLocator.addEventListener(MouseEvent.MOUSE_DOWN, locatorMouseDownHandler);
		}

		/**
		 * Internal mouseDown handler. Mouse pressed on locator
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function locatorMouseDownHandler(event : MouseEvent) : void
		{
			_isDragging = true;
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, locatorMouseMoveHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, locatorMouseUpHandler);
			stage.addEventListener(Event.MOUSE_LEAVE, locatorMouseUpHandler);
			
			_activeLocator = event.target as BasicLocator;
			
			if(_activeLocator == _startLocator) _startLocator.startDrag(false, new Rectangle(_track.x, _track.y, Math.min((_endLocator.x - _track.x), _track.width), 0));
			else if(_activeLocator == _endLocator) _endLocator.startDrag(false, new Rectangle(Math.max(_track.x, _startLocator.x), _track.y, (_track.width - _startLocator.x + _track.x), 0));
		}

		/**
		 * 
		 */
		private function locatorMouseMoveHandler(event : MouseEvent) : void
		{
			if(_isDragging)
			{
				draw();
				
				if(_activeLocator == _startLocator)
				{
					_startValue = _minimum + ((_maximum - _minimum) * ((_startLocator.x - _track.x) / _track.width));
					dispatchEvent(new Event(START_LOCATOR_CHANGE));
				}
				else if(_activeLocator == _endLocator)
				{
					_endValue = _minimum + ((_maximum - _minimum) * ((_endLocator.x - _track.x) / _track.width));
					dispatchEvent(new Event(END_LOCATOR_CHANGE));
				}
			}
		}

		/**
		 * 
		 */
		private function locatorMouseUpHandler(event : Event) : void
		{
			_isDragging = false;
			
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, locatorMouseMoveHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, locatorMouseUpHandler);
			stage.removeEventListener(Event.MOUSE_LEAVE, locatorMouseUpHandler);
			
			_activeLocator.stopDrag();
			_activeLocator = null;
		}
	}
}
