package com.hisschemoller.view.components.ui 
{
	import com.bit101.components.Slider;
	import com.hisschemoller.view.components.enum.Colors;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 21, 2010
	 */
	public class BasicScrollSlider extends BasicSlider
	{
		public static const PAGE_DOWN : String = "pageDown";
		public static const PAGE_UP : String = "pageUp";
		protected var _thumbPercent : Number = 1.0;

		/**
		 * Constructor
		 * @param orientation Whether this is a vertical or horizontal slider.
		 * @param parent The parent DisplayObjectContainer on which to add this Slider.
		 * @param xpos The x position to place this component.
		 * @param ypos The y position to place this component.
		 * @param defaultHandler The event handling function to handle the default event for this component (change in this case).
		 */
		public function BasicScrollSlider(orientation : String, parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0, defaultHandler : Function = null)
		{
			super(orientation, parent, xpos, ypos);
			if(defaultHandler != null)
			{
				addEventListener(Event.CHANGE, defaultHandler);
			}
		}

		/**
		 * Initializes the component.
		 */
		override protected function init() : void
		{
			super.init();
			setSliderParams(1, 1, 0);
			backClick = true;
		}

		/**
		 * Draws the handle of the slider.
		 */
		override protected function drawHandle() : void
		{
			_handle.graphics.clear();
			_handle.graphics.beginFill(Colors.BLOOD_RED);
			
			if(_orientation == HORIZONTAL)
			{
				_handle.graphics.drawRect(0, 0, _width * _thumbPercent, _height);
			}
			else
			{
				_handle.graphics.drawRect(0, 0, _width, _height * _thumbPercent);
			}
			
			positionHandle();
		}

		/**
		 * Adjusts position of handle when value, maximum or minimum have changed.
		 * TODO: Should also be called when slider is resized.
		 */
		override protected function positionHandle() : void
		{
			if(_min == _max)
			{
				if(_orientation == HORIZONTAL) _handle.x = 0;
				else _handle.y = 0;
				return;
			}
			
			if(_orientation == HORIZONTAL)
			{
				var range : Number = width - _handle.width;
				_handle.x = (_value - _min) / (_max - _min) * range;
			}
			else
			{
				range = height - _handle.height;
				_handle.y = (_value - _min) / (_max - _min) * range;
			}
		}

		///////////////////////////////////
		// public methods
		///////////////////////////////////
	
		/**
		 * Sets the percentage of the size of the thumb button.
		 */
		public function setThumbPercent(value : Number) : void
		{
			_thumbPercent = value;
			
			if(_orientation == HORIZONTAL && _thumbPercent * _width < _height)
			{
				_thumbPercent = _height / _width;
			}
			else if(_orientation == VERTICAL && _thumbPercent * _height < _width)
			{
				_thumbPercent = _width / _height;
			}
			
			invalidate();
		}

		/**
		 * Handler called when user clicks the background of the slider, causing the handle to move to that point. Only active if backClick is true.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onBackClick(event : MouseEvent) : void
		{
			if(_orientation == Slider.HORIZONTAL)
			{
				if(mouseX > _handle.x + _handle.width) dispatchEvent(new Event(PAGE_DOWN));
				else if(mouseX < _handle.x) dispatchEvent(new Event(PAGE_UP));
			}
			else
			{
				if(mouseY > _handle.y + _handle.height) dispatchEvent(new Event(PAGE_DOWN));
				else if(mouseY < _handle.y) dispatchEvent(new Event(PAGE_UP));
			}
		}

		/**
		 * Internal mouseMove handler for when the handle is being moved.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onSlide(event : MouseEvent) : void
		{
			if(_min == _max)
			{
				var newValue : Number = 0;
			}
			else if(_orientation == Slider.HORIZONTAL)
			{
				newValue = _handle.x / (_back.width - _handle.width) * (_max - _min) + _min;
			}
			else
			{
				newValue = _handle.y / (_back.height - _handle.height) * (_max - _min) + _min;
			}
			
			doSetValue(newValue, CLICK_TARGET_THUMB);
		}
	}
}
