package com.hisschemoller.view.components.ui 
{
	import com.bit101.components.Component;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Apr 5, 2010
	 */
	public class PlayLocatorBar extends Component 
	{
		public static const PLAY_LOCATOR_CHANGE : String = "playLocatorChange";
		protected var _playLocator : PlayLocator;
		protected var _minimum : Number = 0;
		protected var _maximum : Number = 1;
		protected var _value : Number = 0;
		protected var _isDragging : Boolean;
		
		public function PlayLocatorBar(parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0)
		{
			super(parent, xpos, ypos);
		}

		/**
		 * Sets the size of the component.
		 * @param w The width of the component.
		 * @param h The height of the component.
		 */
		override public function setSize(w : Number, h : Number) : void
		{
			_playLocator.setSize(w, h);
			super.setSize(w, h);
		}
		
		/**
		 * 
		 */
		override public function draw() : void
		{
			if(!_isDragging) _playLocator.x = _value * _width;
			
			if((_value < _minimum) || (_value > _maximum)) _playLocator.visible = false;
			else _playLocator.visible = true;
		}
		
		/**
		 * 
		 */
		public function get value() : Number
		{
			return _value;
		}

		/**
		 * @param inValue : new value for the start locator
		 */
		public function set value(value : Number) : void
		{
			if(_isDragging) return;
			
			_value = value;
			
			invalidate();
		}

		/**
		 * Creates and adds the child display objects of this component.
		 */
		override protected function addChildren() : void
		{
			_playLocator = new PlayLocator(this, 0, 0);
			_playLocator.addEventListener(MouseEvent.MOUSE_DOWN, locatorMouseDownHandler);
		}

		/**
		 * Internal mouseDown handler. Mouse pressed on locator
		 * @param event : The MouseEvent passed by the system.
		 */
		protected function locatorMouseDownHandler(event : MouseEvent) : void
		{
			_isDragging = true;
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, locatorMouseMoveHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, locatorMouseUpHandler);
			stage.addEventListener(Event.MOUSE_LEAVE, locatorMouseUpHandler);
			
			_playLocator.startDrag(false, new Rectangle(0, 0, _width, 0));
		}

		/**
		 * 
		 */
		private function locatorMouseMoveHandler(event : MouseEvent) : void
		{
			if(_isDragging)
			{
				draw();
				
				_value = _minimum + ((_maximum - _minimum) * (_playLocator.x / _width));
				dispatchEvent(new Event(PLAY_LOCATOR_CHANGE));
			}
		}

		/**
		 * 
		 */
		private function locatorMouseUpHandler(event : Event) : void
		{
			_isDragging = false;
			
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, locatorMouseMoveHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, locatorMouseUpHandler);
			stage.removeEventListener(Event.MOUSE_LEAVE, locatorMouseUpHandler);
			
			_playLocator.stopDrag();
		}
	}
}
